from NuRadioReco.modules.base.module import register_run
import h5py
import NuRadioReco.framework.event
import NuRadioReco.framework.station
import NuRadioReco.framework.radio_shower
from radiotools import helper as hp
from radiotools import coordinatesystems as cstrafo
from NuRadioReco.modules.io.coreas import coreas
from NuRadioReco.utilities import units
import numpy as np
import logging
import time
import os



class readCoREAS:

    def __init__(self):
        self.__t = 0
        self.__t_event_structure = 0
        self.__t_per_event = 0
        self.logger = logging.getLogger('NuRadioReco.readCoREAS')

    def begin(self, input_files, station_id, n_cores=10, max_distance=2 * units.km, seed=0):
        """
        begin method

        initialize readCoREAS module

        Parameters
        ----------
        input_files: input files
            list of coreas hdf5 files
        station_id: station id
            id number of the station
        n_cores: number of cores (integer)
            the number of random core positions to generate for each input file
        max_distance: radius of random cores (double or None)
            if None: max distance is set to the maximum ground distance of the
            star pattern simulation
        """
        self.__input_files = input_files
        self.__station_id = station_id
        self.__n_cores = n_cores
        self.__max_distace = max_distance
        self.__current_input_file = 0
        np.random.seed(seed)

    @register_run()
    def run(self, detector, output_mode=0):
        """
        Read in a random sample of stations from a CoREAS file.
        A number of random positions is selected within a certain radius.
        For each position the closest observer is selected and a simulated
        event is created for that observer.

        Parameters
        ----------
        detector: Detector object
            Detector description of the detector that shall be simulated
        output_mode: integer (default 0)
            0: only the event object is returned
            1: the function reuturns the event object, the current inputfilename, the distance between the choosen station and the requested core position,
               and the area in which the core positions are randomly distributed


        """
        while (self.__current_input_file < len(self.__input_files)):
            t = time.time()
            t_per_event = time.time()
            filesize = os.path.getsize(self.__input_files[self.__current_input_file])
            if(filesize < 18456 * 2):
                self.logger.warning("file {} seems to be corrupt, skipping to next file".format(self.__input_files[self.__current_input_file]))
                self.__current_input_file += 1
                continue
            corsika = h5py.File(self.__input_files[self.__current_input_file], "r")
            self.logger.info("using coreas simulation {} with E={:2g} theta = {:.0f}".format(self.__input_files[self.__current_input_file],
                                                                                                      corsika['inputs'].attrs["ERANGE"][0] * units.GeV,
                                                                                                      corsika['inputs'].attrs["THETAP"][0]))
            positions = []
            for i, observer in enumerate(corsika['CoREAS']['observers'].values()):
                position = observer.attrs['position']
                positions.append(np.array([-position[1], position[0], 0]) * units.cm)
                self.logger.debug("({:.0f}, {:.0f})".format(position[0], position[1]))
            positions = np.array(positions)

            max_distance = self.__max_distace
            if(max_distance is None):
                max_distance = np.max(np.abs(positions[:, 0:2]))
            area = np.pi * max_distance ** 2

            if(output_mode == 0):
                n_cores = self.__n_cores * 100  # for output mode 1 we want always n_cores in star pattern. Therefore we generate more core positions to be able to select n_cores in the star pattern afterwards
            elif(output_mode == 1):
                n_cores = self.__n_cores

            theta = np.random.rand(n_cores) * 2 * np.pi
            r = (np.random.rand(n_cores)) ** 0.5 * max_distance
            cores = np.array([r * np.cos(theta), r * np.sin(theta), np.zeros(n_cores)]).T

            zenith, azimuth, magnetic_field_vector = coreas.get_angles(corsika)
            cs = cstrafo.cstrafo(zenith, azimuth, magnetic_field_vector)
            positions_vBvvB = cs.transform_from_magnetic_to_geographic(positions.T)
            positions_vBvvB = cs.transform_to_vxB_vxvxB(positions_vBvvB).T
            dd = (positions_vBvvB[:, 0] ** 2 + positions_vBvvB[:, 1] ** 2) ** 0.5
            ddmax = dd.max()
            self.logger.info("star shape from: {} - {}".format(-dd.max(), dd.max()))

            cores_vBvvB = cs.transform_from_magnetic_to_geographic(cores.T)
            cores_vBvvB = cs.transform_to_vxB_vxvxB(cores_vBvvB).T
            dcores = (cores_vBvvB[:, 0] ** 2 + cores_vBvvB[:, 1] ** 2) ** 0.5
            mask_cores_in_starpattern = dcores <= ddmax

            if((not np.sum(mask_cores_in_starpattern)) and (output_mode == 1)):  # handle special case of no core position being generated within star pattern
                observer = corsika['CoREAS']['observers'].values()[0]

                evt = NuRadioReco.framework.event.Event(corsika['inputs'].attrs['RUNNR'], corsika['inputs'].attrs['EVTNR'])  # create empty event
                station = NuRadioReco.framework.station.Station(self.__station_id)
                sim_station = coreas.make_sim_station(self.__station_id, corsika, observer)

                station.set_sim_station(sim_station)
                evt.set_station(station)
                yield evt, self.__current_input_file, None, area

            cores_to_iterate = cores_vBvvB[mask_cores_in_starpattern]
            if(output_mode == 0):  # select first n_cores that are in star pattern
                if(np.sum(mask_cores_in_starpattern) < self.__n_cores):
                    self.logger.warning("only {0} cores contained in star pattern, returning {0} cores instead of {1} cores that were requested".format(np.sum(mask_cores_in_starpattern), self.__n_cores))
                else:
                    cores_to_iterate = cores_vBvvB[mask_cores_in_starpattern][:self.__n_cores]

            self.__t_per_event += time.time() - t_per_event
            self.__t += time.time() - t

            for iCore, core in enumerate(cores_to_iterate):
                t = time.time()
                # check if out of bounds

                distances = np.linalg.norm(core[:2] - positions_vBvvB[:, :2], axis=1)
                index = np.argmin(distances)
                distance = distances[index]
                key = list(corsika['CoREAS']['observers'].keys())[index]
                self.logger.info("generating core at ground ({:.0f}, {:.0f}), vBvvB({:.0f}, {:.0f}), nearest simulated station is {:.0f}m away at ground ({:.0f}, {:.0f}), vBvvB({:.0f}, {:.0f})".format(cores[iCore][0], cores[iCore][1], core[0], core[1], distance / units.m,
                                                                                                                                        positions[index][0], positions[index][1], positions_vBvvB[index][0], positions_vBvvB[index][1]))
#                 import matplotlib.pyplot as plt
#                 indexes = np.array(range(len(cores)))
#                 fig, ax = plt.subplots(1, 1)
#                 ax.plot(positions_vBvvB[:, 0], positions_vBvvB[:, 1], 'o')
#                 ax.plot(positions[:, 0], positions[:, 1], 'd')
#                 ax.plot(core[0], core[1], '*')
#                 ax.plot(cores[indexes[mask_cores_in_starpattern][iCore]][0], cores[indexes[mask_cores_in_starpattern][iCore]][1], 'x')
#                 ax.plot(positions_vBvvB[index][0], positions_vBvvB[index][1], 'd')
#                 ax.plot(positions[index][0], positions[index][1], 'D')
#                 ax.set_aspect("equal")
#                 ax.set_title("zen {:.0f}, az {:.0f}".format(zenith / units.deg, azimuth / units.deg))
#                 plt.show()

                t_event_structure = time.time()
                observer = corsika['CoREAS']['observers'].get(key)

                evt = NuRadioReco.framework.event.Event(self.__current_input_file, iCore)  # create empty event
                station = NuRadioReco.framework.station.Station(self.__station_id)
                channel_ids = detector.get_channel_ids(self.__station_id)
                sim_station = coreas.make_sim_station(self.__station_id, corsika, observer, channel_ids)
                station.set_sim_station(sim_station)
                evt.set_station(station)
                sim_shower = coreas.make_sim_shower(corsika, observer, detector, self.__station_id)
                evt.add_sim_shower(sim_shower)
                rd_shower = NuRadioReco.framework.radio_shower.RadioShower([station.get_id()])
                evt.add_shower(rd_shower)
                if(output_mode == 0):
                    self.__t += time.time() - t
                    self.__t_event_structure += time.time() - t_event_structure
                    yield evt
                elif(output_mode == 1):
                    self.__t += time.time() - t
                    self.__t_event_structure += time.time() - t_event_structure
                    yield evt, self.__current_input_file, distance, area
                else:
                    self.logger.debug("output mode > 1 not implemented")
                    raise NotImplementedError

            self.__current_input_file += 1

    def end(self):
        from datetime import timedelta
        self.logger.setLevel(logging.INFO)
        dt = timedelta(seconds=self.__t)
        self.logger.info("total time used by this module is {}".format(dt))
        self.logger.info("\tcreate event structure {}".format(timedelta(seconds=self.__t_event_structure)))
        self.logger.info("\per event {}".format(timedelta(seconds=self.__t_per_event)))
        return dt



from NuRadioReco.modules.base.module import register_run
import numpy as np
import logging
import time

from NuRadioReco.utilities import units


class simulationSelector:

    '''
    Module that let's you select CoREAS simulations
    based on certain criteria, e.g. signal in a relevant band
    certain arrival directions, energies, etc.
    '''

    def __init__(self):
        self.__t = 0
        self.begin()
        self.logger = logging.getLogger('NuRadioReco.simulationSelector')

    def begin(self, debug=False):
        pass

    @register_run()
    def run(self, evt, sim_station, det, frequency_window=[100 * units.MHz, 500 * units.MHz], n_std=8):

        """
        run method, selects CoREAS simulations that have any signal in
        desired frequency_window.
        Crude approximation with n_std sigma * noise

        Parameters
        ------------
        evt: Event
        sim_station: sim_station
            CoREAS simulated efields
        det: Detector
        frequency_window: list
            [lower, upper] frequencies that will be used for analysis
        n_std: int
            number of std deviations needed, can make cut stricter, if needed

        Returns
        ------------
        selected_sim: bool
            if True then simulation has signal in desired range

        """
        t = time.time()
        efields = sim_station.get_electric_fields()
        selected_sim = False
        j = 0
        for efield in efields:
            fft = np.abs(efield.get_frequency_spectrum())
            freq = efield.get_frequencies()

            # identify the largest polarization
            max_pol = 0
            max_ = 0
            for i in range(3):
                if np.sum(fft[i]) > max_:
                    max_pol = i
                    max_ = np.sum(fft[i])

            # Find a n_std sigma excess above the noise
            # Seems to be a reasonably well-working number

            noise_region = fft[max_pol][np.where(freq > 1.5 * units.GHz)]
            noise = np.mean(noise_region)
            if noise == 0:
                self.logger.warning("Trace seems to have bee upsampled beyong 1.5 GHz, using lower noise window")
                noise_region = fft[max_pol][np.where(freq > 1. * units.GHz)]
                noise = np.mean(noise_region)
            if noise == 0:
                self.logger.warning("Trace seems to have bee upsampled beyong 1. GHz, using lower noise window")
                noise_region = fft[max_pol][np.where(freq > 800. * units.MHz)]
                noise = np.mean(noise_region)
            if noise == 0:
                self.logger.error("Trace seems to have bee upsampled beyong 800 MHz, unsuitable simulations")

            noise_std = np.std(noise_region)

            noise += n_std * noise_std

            mask = np.array((freq >= np.min(frequency_window)) & (freq <= np.max(frequency_window)))

            if(np.any(fft[:, mask] > noise)):
                selected_sim = True
                break

        self.__t += time.time() - t
        return selected_sim

    def end(self):
        from datetime import timedelta
        self.logger.setLevel(logging.INFO)
        dt = timedelta(seconds=self.__t)
        self.logger.info("total time used by this module is {}".format(dt))
        return dt



import NuRadioReco.framework.event
import NuRadioReco.framework.station
from NuRadioReco.framework.parameters import showerParameters as shp
from NuRadioReco.modules.io.coreas import coreas
from NuRadioReco.utilities import units
from radiotools import coordinatesystems

import h5py
import numpy as np
import time
import re
import os

import logging
logger = logging.getLogger('readCoREASShower')


class readCoREASShower:

    def __init__(self):
        self.__t = 0
        self.__t_event_structure = 0
        self.__t_per_event = 0

    def begin(self, input_files, det=None, logger_level=logging.NOTSET, set_ascending_run_and_event_number=False):
        """
        begin method

        initialize readCoREASShower module

        Parameters
        ----------
        input_files: input files
            list of coreas hdf5 files
        det: genericDetector object
            If a genericDetector is passed, the stations from the CoREAS file
            will be added to it and the run method returns both the event and
            the detector
        logger_level: string or logging variable
            Set verbosity level for logger (default: logging.NOTSET)
        set_ascending_run_and_event_number: bool
            If set to True the run number and event id is set to
            self.__ascending_run_and_event_number instead of beeing taken
            from the simulation file. The value is increases monoton.
            This can be used to avoid ambiguities values (default: False)
        """
        self.__input_files = input_files
        self.__current_input_file = 0
        self.__det = det
        logger.setLevel(logger_level)

        self.__ascending_run_and_event_number = 1 if set_ascending_run_and_event_number else 0


    def run(self):
        """
        Reads in a CoREAS file and returns an event containing all simulated stations

        """
        while (self.__current_input_file < len(self.__input_files)):
            t = time.time()
            t_per_event = time.time()

            filesize = os.path.getsize(self.__input_files[self.__current_input_file])
            if(filesize < 18456 * 2):
                logger.warning("file {} seems to be corrupt, skipping to next file".format(
                       self.__input_files[self.__current_input_file]))
                self.__current_input_file += 1
                continue

            logger.info('Reading %s ...' % self.__input_files[self.__current_input_file])

            corsika = h5py.File(self.__input_files[self.__current_input_file], "r")
            logger.info("using coreas simulation {} with E={:2g} theta = {:.0f}".format(
                self.__input_files[self.__current_input_file], corsika['inputs'].attrs["ERANGE"][0] * units.GeV,
                corsika['inputs'].attrs["THETAP"][0]))

            f_coreas = corsika["CoREAS"]

            if self.__ascending_run_and_event_number:
                evt = NuRadioReco.framework.event.Event(self.__ascending_run_and_event_number,
                                                        self.__ascending_run_and_event_number)
                self.__ascending_run_and_event_number += 1
            else:
                evt = NuRadioReco.framework.event.Event(corsika['inputs'].attrs['RUNNR'], corsika['inputs'].attrs['EVTNR'])

            evt.__event_time = f_coreas.attrs["GPSSecs"]

            # create sim shower, no core is set since no external detector description is given
            sim_shower = coreas.make_sim_shower(corsika)
            sim_shower.set_parameter(shp.core, np.array([0, 0, f_coreas.attrs["CoreCoordinateVertical"] / 100])) # set core
            evt.add_sim_shower(sim_shower)

            # initialize coordinate transformation
            cs = coordinatesystems.cstrafo(sim_shower.get_parameter(shp.zenith), sim_shower.get_parameter(shp.azimuth),
                                           magnetic_field_vector=sim_shower.get_parameter(shp.magnetic_field_vector))

            # add simulated pulses as sim station
            for idx, (name, observer) in enumerate(f_coreas['observers'].items()):
                station_id = antenna_id(name, idx)  # returns proper station id if possible

                station = NuRadioReco.framework.station.Station(station_id)
                sim_station = coreas.make_sim_station(station_id, corsika, observer, channel_ids=[0, 1, 2])

                station.set_sim_station(sim_station)
                evt.set_station(station)
                if self.__det is not None:
                    position = observer.attrs['position']
                    antenna_position = np.zeros(3)
                    antenna_position[0], antenna_position[1], antenna_position[2] = -position[1] * units.cm, position[0] * units.cm, position[2] * units.cm
                    antenna_position = cs.transform_from_magnetic_to_geographic(antenna_position)
                    if not self.__det.has_station(station_id):
                        self.__det.add_generic_station({
                            'station_id': station_id,
                            'pos_easting': antenna_position[0],
                            'pos_northing': antenna_position[1],
                            'pos_altitude': antenna_position[2]
                        })
                    else:
                        self.__det.add_station_properties_for_event({
                            'pos_easting': antenna_position[0],
                            'pos_northing': antenna_position[1],
                            'pos_altitude': antenna_position[2]
                        }, station_id, evt.get_run_number(), evt.get_id())

            self.__t_per_event += time.time() - t_per_event
            self.__t += time.time() - t

            self.__current_input_file += 1
            if self.__det is None:
                yield evt
            else:
                self.__det.set_event(evt.get_run_number(), evt.get_id())
                yield evt, self.__det

    def end(self):
        from datetime import timedelta
        logger.setLevel(logging.INFO)
        dt = timedelta(seconds=self.__t)
        logger.info("total time used by this module is {}".format(dt))
        logger.info("\tcreate event structure {}".format(timedelta(seconds=self.__t_event_structure)))
        logger.info("\per event {}".format(timedelta(seconds=self.__t_per_event)))
        return dt


def antenna_id(antenna_name, default_id):
    """
    This function parses the antenna names given in a CoREAS simulation and tries to find an ID
    It can be extended to other name patterns
    """

    if re.match("AERA_", antenna_name):
        new_id = int(antenna_name.strip("AERA_"))
        return new_id
    else:
        return default_id



from NuRadioReco.modules.base.module import register_run
import numpy as np
import time

from NuRadioReco.utilities import ice

from NuRadioReco.framework.parameters import stationParameters as stnp
from NuRadioReco.framework.parameters import channelParameters as chp

import logging
logger = logging.getLogger('efieldAirToIcePropagator')


class efieldAirToIcePropagator:
    """
    Module that propagates the efield (usually from air showers) into the ice.
    """

    def __init__(self):
        self.__t = 0
        self.begin()

    def begin(self, debug=False):
        self.__debug = debug

    @register_run()
    def run(self, evt, station, det):
        t = time.time()

        logger.warning("Nothing implemented yet")

        self.__t += time.time() - t

    def end(self):
        from datetime import timedelta
        logger.setLevel(logging.INFO)
        dt = timedelta(seconds=self.__t)
        logger.info("total time used by this module is {}".format(dt))
        return dt



import dash_html_components as html
import numbers

def get_point_index(event_ids, selection):
    index = {}
    for i, event_id in enumerate(event_ids):
        if(not isinstance(event_id, basestring)):
            event_id = str(event_id)
        index[event_id] = i
    event_index = []
    for event_id in selection:
        if event_id in index.keys():
            event_index.append(index[event_id])
    return event_index

def get_properties_divs(obj, props_dic):
    props = []
    for display_prop in props_dic:
        if obj.has_parameter(display_prop['param']):
            if type(obj.get_parameter(display_prop['param'])) is dict:
                dict_entries = []
                dic = obj.get_parameter(display_prop['param'])
                for key in dic:
                    if isinstance(dic[key], numbers.Number):
                        dict_entries.append(
                            html.Div([
                                html.Div(key, className='custom-table-td'),
                                html.Div('{:.2f}'.format(dic[key]), className='custom-table-td custom-table-td-last')
                            ], className='custom-table-row')
                        )
                prop = html.Div(dict_entries, className='custom-table-td')
            else:
                if display_prop['unit'] is not None:
                    v = obj.get_parameter(display_prop['param'])/display_prop['unit']
                else:
                    v = obj.get_parameter(display_prop['param'])
                if isinstance(v,float) or isinstance(v, int):
                    prop = html.Div('{:.2f}'.format(v), className='custom-table-td custom-table-td-last')
                else:
                    prop = html.Div('{}'.format(v), className='custom-table-td custom-table-td-last')
            props.append(html.Div([
                html.Div(display_prop['label'], className='custom-table-td'),
                prop
            ], className='custom-table-row'))
    return props



import numpy as np
import scipy
from NuRadioReco.utilities import units
import NuRadioReco.framework.sim_station
from NuRadioReco.framework.parameters import stationParameters as stnp
from NuRadioReco.framework.parameters import channelParameters as chp
from NuRadioReco.utilities import ice
from NuRadioReco.utilities import geometryUtilities as geo_utl
from NuRadioReco.utilities import fft
import logging
logger = logging.getLogger('NuRadioReco.trace_utilities')

conversion_factor_integrated_signal = scipy.constants.c * scipy.constants.epsilon_0 * units.joule / units.s / units.volt ** 2

# see Phys. Rev. D DOI: 10.1103/PhysRevD.93.122005
# to convert V**2/m**2 * s -> J/m**2 -> eV/m**2


def get_efield_antenna_factor(station, frequencies, channels, detector, zenith, azimuth, antenna_pattern_provider):
    """
    Returns the antenna response to a radio signal coming from a specific direction

    Parameters
    ---------------
    station: Station
    frequencies: array of complex
        frequencies of the radio signal for which the antenna response is needed
    channels: array of int
        IDs of the channels
    detector: Detector
    zenith, azimuth: float, float
        incoming direction of the signal. Note that refraction and reflection at the ice/air boundary are taken into account
    antenna_pattern_provider: AntennaPatternProvider
    """
    n_ice = ice.get_refractive_index(-0.01, detector.get_site(station.get_id()))
    efield_antenna_factor = np.zeros((len(channels), 2, len(frequencies)), dtype=np.complex)  # from antenna model in e_theta, e_phi
    for iCh, channel_id in enumerate(channels):
        zenith_antenna = zenith
        t_theta = 1.
        t_phi = 1.
        # first check case if signal comes from above
        if zenith <= 0.5 * np.pi and station.is_cosmic_ray():
            # is antenna below surface?
            position = detector.get_relative_position(station.get_id(), channel_id)
            if position[2] <= 0:
                zenith_antenna = geo_utl.get_fresnel_angle(zenith, n_ice, 1)
                t_theta = geo_utl.get_fresnel_t_p(zenith, n_ice, 1)
                t_phi = geo_utl.get_fresnel_t_s(zenith, n_ice, 1)
                logger.info("channel {:d}: electric field is refracted into the firn. theta {:.0f} -> {:.0f}. Transmission coefficient p (eTheta) {:.2f} s (ePhi) {:.2f}".format(iCh, zenith / units.deg, zenith_antenna / units.deg, t_theta, t_phi))
        else:
            # now the signal is coming from below, do we have an antenna above the surface?
            position = detector.get_relative_position(station.get_id(), channel_id)
            if(position[2] > 0):
                zenith_antenna = geo_utl.get_fresnel_angle(zenith, 1., n_ice)
        if(zenith_antenna is None):
            logger.warning("fresnel reflection at air-firn boundary leads to unphysical results, no reconstruction possible")
            return None

        logger.debug("angles: zenith {0:.0f}, zenith antenna {1:.0f}, azimuth {2:.0f}".format(np.rad2deg(zenith), np.rad2deg(zenith_antenna), np.rad2deg(azimuth)))
        antenna_model = detector.get_antenna_model(station.get_id(), channel_id, zenith_antenna)
        antenna_pattern = antenna_pattern_provider.load_antenna_pattern(antenna_model)
        ori = detector.get_antenna_orientation(station.get_id(), channel_id)
        VEL = antenna_pattern.get_antenna_response_vectorized(frequencies, zenith_antenna, azimuth, *ori)
        efield_antenna_factor[iCh] = np.array([VEL['theta'] * t_theta, VEL['phi'] * t_phi])
    return efield_antenna_factor


def get_channel_voltage_from_efield(station, electric_field, channels, detector, zenith, azimuth, antenna_pattern_provider, return_spectrum=True):
    """
    Returns the voltage traces that would result in the channels from the station's E-field.

    Parameters
    ------------------------
    station: Station
    electric_field: ElectricField
    channels: array of int
        IDs of the channels for which the expected voltages should be calculated
    detector: Detector
    zenith, azimuth: float
        incoming direction of the signal. Note that reflection and refraction
        at the air/ice boundary are already being taken into account.
    antenna_pattern_provider: AntennaPatternProvider
    return_spectrum: boolean
        if True, returns the spectrum, if False return the time trace
    """

    frequencies = electric_field.get_frequencies()
    spectrum = electric_field.get_frequency_spectrum()
    efield_antenna_factor = get_efield_antenna_factor(station, frequencies, channels, detector, zenith, azimuth, antenna_pattern_provider)
    if return_spectrum:
        voltage_spectrum = np.zeros((len(channels), len(frequencies)), dtype=np.complex)
        for i_ch, ch in enumerate(channels):
            voltage_spectrum[i_ch] = np.sum(efield_antenna_factor[i_ch] * np.array([spectrum[1], spectrum[2]]), axis=0)
        return voltage_spectrum
    else:
        voltage_trace = np.zeros((len(channels), 2 * (len(frequencies) - 1)), dtype=np.complex)
        for i_ch, ch in enumerate(channels):
            voltage_trace[i_ch] = fft.freq2time(np.sum(efield_antenna_factor[i_ch] * np.array([spectrum[1], spectrum[2]]), axis=0), electric_field.get_sampling_rate())
        return np.real(voltage_trace)


def get_electric_field_energy_fluence(electric_field_trace, times, signal_window_mask=None, noise_window_mask=None):

    if signal_window_mask is None:
        f_signal = np.sum(electric_field_trace ** 2, axis=1)
    else:
        f_signal = np.sum(electric_field_trace[:, signal_window_mask] ** 2, axis=1)
    dt = times[1] - times[0]
    if noise_window_mask is not None:
        f_noise = np.sum(electric_field_trace[:, noise_window_mask] ** 2, axis=1)
        f_signal -= f_noise * np.sum(signal_window_mask) / np.sum(noise_window_mask)

    return f_signal * dt * conversion_factor_integrated_signal



from __future__ import absolute_import, division, print_function
import NuRadioReco.framework.base_station
import NuRadioReco.framework.sim_station
import NuRadioReco.framework.channel
from six import iteritems
import pickle
import logging
import collections
logger = logging.getLogger('Station')


class Station(NuRadioReco.framework.base_station.BaseStation):

    def __init__(self, station_id):
        NuRadioReco.framework.base_station.BaseStation.__init__(self, station_id)
        self.__channels = collections.OrderedDict()
        self.__reference_reconstruction = 'RD'
        self.__sim_station = None

    def set_sim_station(self, sim_station):
        self.__sim_station = sim_station

    def get_sim_station(self):
        return self.__sim_station

    def has_sim_station(self):
        return self.__sim_station is not None

    def iter_channels(self, use_channels=None):
        for channel_id, channel in iteritems(self.__channels):
            if(use_channels is None):
                yield channel
            else:
                if channel_id in use_channels:
                    yield channel

    def get_channel(self, channel_id):
        return self.__channels[channel_id]

    def get_number_of_channels(self):
        return len(self.__channels)

    def add_channel(self, channel):
        self.__channels[channel.get_id()] = channel

    def set_reference_reconstruction(self, reference):
        if reference not in ['RD', 'MC']:
            import sys
            logger.error("reference reconstructions other than RD and MC are not supported")
            sys.exit(-1)
        self.__reference_reconstruction = reference

    def get_reference_reconstruction(self):
        return self.__reference_reconstruction

    def get_reference_direction(self):
        if(self.__reference_reconstruction == 'RD'):
            return self.get_parameter('zenith'), self.get_parameter('azimuth')
        if(self.__reference_reconstruction == 'MC'):
            return self.get_sim_station().get_parameter('zenith'), self.get_sim_station().get_parameter('azimuth')

    def get_magnetic_field_vector(self, time=None):
        if(self.__reference_reconstruction == 'MC'):
            return self.get_sim_station().get_magnetic_field_vector()
        if(self.__reference_reconstruction == 'RD'):
            if time is not None:
                logger.warning("time dependent magnetic field model not yet implemented, returning static magnetic field for the ARIANNA site")
            from radiotools import helper
            return helper.get_magnetic_field_vector('arianna')

    def serialize(self, mode):
        base_station_pkl = NuRadioReco.framework.base_station.BaseStation.serialize(self, mode)
        channels_pkl = []
        for channel in self.iter_channels():
            channels_pkl.append(channel.serialize(mode))
        sim_station_pkl = None
        if(self.has_sim_station()):
            sim_station_pkl = self.get_sim_station().serialize(mode)

        data = {'__reference_reconstruction': self.__reference_reconstruction,
                'channels': channels_pkl,
                'base_station': base_station_pkl,
                'sim_station': sim_station_pkl}
        return pickle.dumps(data, protocol=4)

    def deserialize(self, data_pkl):
        data = pickle.loads(data_pkl)
        NuRadioReco.framework.base_station.BaseStation.deserialize(self, data['base_station'])
        if(data['sim_station'] is None):
            self.__sim_station = None
        else:
            self.__sim_station = NuRadioReco.framework.sim_station.SimStation(None, None, None)
            self.__sim_station.deserialize(data['sim_station'])
        for channel_pkl in data['channels']:
            channel = NuRadioReco.framework.channel.Channel(0)
            channel.deserialize(channel_pkl)
            self.add_channel(channel)

        self.__reference_reconstruction = data['__reference_reconstruction']



from __future__ import absolute_import, division, print_function
import NuRadioReco.framework.base_station
import numpy as np
from six import iteritems
try:
    import cPickle as pickle
except ImportError:
    import pickle
import logging
logger = logging.getLogger('SimStation')


class SimStation(NuRadioReco.framework.base_station.BaseStation):

    def __init__(self, station_id, sampling_rate=None, trace=None):
        NuRadioReco.framework.base_station.BaseStation.__init__(self, station_id)
        self.__magnetic_field_vector = None
        self.__simulation_weight = None
        if(trace is not None and sampling_rate is not None):
            self.set_electric_fields(trace, sampling_rate)

    def get_magnetic_field_vector(self):
        return self.__magnetic_field_vector

    def set_magnetic_field_vector(self, magnetic_field_vector):
        self.__magnetic_field_vector = magnetic_field_vector

    def get_simulation_weight(self):
        return self.__simulation_weight

    def set_simulation_weight(self, simulation_weight):
        self.__simulation_weight = simulation_weight

    def serialize(self, mode):
        base_station_pkl = NuRadioReco.framework.base_station.BaseStation.serialize(self, mode)
        data = {'__magnetic_field_vector': self.__magnetic_field_vector,
                '__simulation_weight': self.__simulation_weight,
                'base_station': base_station_pkl}
        return pickle.dumps(data, protocol=4)

    def deserialize(self, data_pkl):
        data = pickle.loads(data_pkl)
        NuRadioReco.framework.base_station.BaseStation.deserialize(self, data['base_station'])
        self.__magnetic_field_vector = data['__magnetic_field_vector']
        self.__simulation_weight = data['__simulation_weight']
        



import numpy as np
from NuRadioReco.modules.base.module import register_run
from NuRadioReco.utilities import units
import scipy.signal


class electricFieldBandPassFilter:
    """
    """

    def begin(self):
        pass

    @register_run()
    def run(self, evt, station, det, passband=[55 * units.MHz, 1000 * units.MHz],
            filter_type='rectangular', order=2, debug=False):
        """
        Applies bandpass filter to electric field

        Parameters
        ----------
        evt: event

        station: station

        det: detector

        passband: seq, array
            passband for filter, in phys units
        filter_type: string
            'rectangular': perfect straight line filter
            'butter': butterworth filter from scipy
            'butterabs': absolute of butterworth filter from scipy
        order: int (optional, default 2)
            for a butterworth filter: specifies the order of the filter
        debug: bool
            set debug

        """

        for efield in station.get_electric_fields():
            frequencies = efield.get_frequencies()
            trace_fft = efield.get_frequency_spectrum()
            if(filter_type == 'rectangular'):
                trace_fft[:, np.where(frequencies < passband[0])] = 0.
                trace_fft[:, np.where(frequencies > passband[1])] = 0.
            elif(filter_type == 'butter'):
                b, a = scipy.signal.butter(order, passband, 'bandpass', analog=True)
                w, h = scipy.signal.freqs(b, a, frequencies)
                trace_fft *= h
            elif(filter_type == 'butterabs'):
                b, a = scipy.signal.butter(order, passband, 'bandpass', analog=True)
                w, h = scipy.signal.freqs(b, a, frequencies)
                trace_fft *= np.abs(h)
            efield.set_frequency_spectrum(trace_fft, efield.get_sampling_rate())

    def end(self):
        pass



from __future__ import absolute_import, division, print_function, unicode_literals
from NuRadioReco.modules.base.module import register_run
from NuRadioReco.modules.io import NuRadioRecoio
import logging
logger = logging.getLogger('eventReader')


class eventReader:
    """
    read events from file
    """

    def begin(self, filename, read_detector=False, log_level=logging.WARNING):
        """
        Setup function for the eventReader module

        Parameters
        -----------
        filename: array if strings
            names of the input files
        read_detector: boolean
            If True, the eventReader will parse the detector description and event event headers
             in the event files. This is necessary to use the get_detector functions
        log_level: logging enum
        """

        self.__fin = NuRadioRecoio.NuRadioRecoio(filename, parse_header=read_detector, log_level=log_level)

    @register_run()
    def run(self):
        return self.__fin.get_events()

    def end(self):
        self.__fin.close_files()

    def get_header(self):
        """
        returns the header information of all events, useful to get a quick overview of all events without
        looping through all events
        """
        return self.__fin.get_header()

    def get_detector(self):
        """
        If read_detector was set True in the begin() function, this function return
        the detector description (assuming there is one in the files). If several
        files with different detectors are read, the detector for the last returned
        event is given.
        """
        return self.__fin.get_detector()



import logging
import numpy as np
import datetime
from NuRadioReco.utilities import units
import NuRadioReco.detector.detector as detector

# Logging level
from NuRadioReco.modules.base import module
logger = module.setup_logger(level=logging.INFO)

# read in detector positions (this is a dummy detector)
det = detector.Detector()

station_id = 52

det.update(datetime.datetime(2018, 1, 1))

n_channels = det.get_number_of_channels(station_id)

for channel_id in range(n_channels):
    position = det.get_relative_position(station_id, channel_id)
    antenna_orientation = det.get_antenna_orientation(station_id, channel_id)
    print("position of channel {:d} is {:.2f} m, {:.2f} m, {:.2f} ".format(channel_id, *position / units.m))
    print("antenna boresight direction ({:.0f}, {:.0f}), antenna rotation ({:.0f}, {:.0f})".format(*antenna_orientation / units.deg))




import NuRadioReco.framework.base_shower
import pickle

class HybridShower(NuRadioReco.framework.base_shower.BaseShower):
    def __init__(self, name):
        self.__name = name
        self.__hybrid_detector = None
        super().__init__()

    def get_name(self):
        return self.__name

    def set_hybrid_detector(self, hybrid_detector):
        self.__hybrid_detector = hybrid_detector

    def get_hybrid_detector(self):
        return self.__hybrid_detector

    def serialize(self):
        base_shower_pickle = NuRadioReco.framework.base_shower.BaseShower.serialize(self)
        if self.__hybrid_detector is not None:
            det_pickle = self.__hybrid_detector.serialize()
            det_class = self.__hybrid_detector.__class__
            detector_info = {
                'data': det_pickle,
                'class': det_class
            }
        else:
            detector_info=None
        data = {
            'base_shower': base_shower_pickle,
            'name': self.__name,
            'detector': detector_info
        }
        return pickle.dumps(data, protocol=4)

    def deserialize(self, data_pkl):
        data = pickle.loads(data_pkl)
        NuRadioReco.framework.base_shower.BaseShower.deserialize(self, data['base_shower'])
        self.__name = data['name']
        if data['detector'] is not None:
            det = data['detector']['class']()
            det.deserialize(data['detector']['data'])
            self.set_hybrid_detector(det)



from NuRadioReco.modules.base.module import register_run
import numpy as np
import fractions
from scipy import signal
from decimal import Decimal
import copy
from NuRadioReco.utilities import units, fft
import NuRadioReco.framework.sim_station
import logging



class channelResampler:
    """
    Resamples the trace to a new sampling rate.
    """

    def __init__(self):
        self.logger = logging.getLogger('NuRadioReco.channelResampler')
        self.begin()

    def begin(self, debug=False, log_level=logging.WARNING):
        self.__max_upsampling_factor = 5000
        self.__debug = debug
        self.logger.setLevel(log_level)

        """
        Begin the channelResampler

        Parameters
        ---------

        __debug: bool
            Debug switch

        """

    @register_run()
    def run(self, evt, station, det, sampling_rate):
        """
        Run the channelResampler

        Parameters
        ---------

        evt, station, det
            Event, Station, Detector
        sampling_rate: float
            In units 1/time provides the desired sampling rate of the data.

        """
        is_sim_station = isinstance(station, NuRadioReco.framework.sim_station.SimStation)
        if is_sim_station: # sim_stations are structured differently
            orig_binning = 1. / station.get_channel(0)[0].get_sampling_rate()  # assume that all channels have the same sampling rate
        else:
            orig_binning = 1. / station.get_channel(0).get_sampling_rate()
        target_binning = 1. / sampling_rate
        resampling_factor = fractions.Fraction(Decimal(orig_binning / target_binning)).limit_denominator(self.__max_upsampling_factor)
        if resampling_factor == self.__max_upsampling_factor:
            self.logger.warning("Safeguard caught, max upsampling {} factor reached.".format(self.__max_upsampling_factor))
        self.logger.debug("resampling channel trace by {}. Original binning is {:.3g} ns, target binning is {:.3g} ns".format(resampling_factor,
                                                                                                                         orig_binning / units.ns,
                                                                                                                         target_binning / units.ns))
        for channel in station.iter_channels():
            trace = channel.get_trace()
            if(self.__debug):
                ff = np.fft.rfftfreq(len(trace), orig_binning)
                spec = fft.time2freq(trace, channel.get_sampling_rate())
                spec[ff >= 500 * units.MHz] = 0
                trace = fft.freq2time(spec, channel.get_sampling_rate())
                trace_old = copy.copy(trace)
            if(resampling_factor.numerator != 1):
                trace = signal.resample(trace, resampling_factor.numerator * len(trace))  # , window='hann')
            if(resampling_factor.denominator != 1):
                trace = signal.resample(trace, len(trace) // resampling_factor.denominator)  # , window='hann')

            # make sure that trace has even number of samples
            if(len(trace) % 2 != 0):
                self.logger.info("channel trace has a odd number of samples after resampling. The last bin of the trace is discarded to maintain a even number of samples")
                trace = trace[:-1]

            if(self.__debug):

                if(resampling_factor.denominator != 1):
                    trace2 = signal.resample(trace, resampling_factor.denominator * len(trace))  # , window='hann')
                if(resampling_factor.numerator != 1):
                    trace2 = signal.resample(trace, len(trace) // resampling_factor.numerator)  # , window='hann')

                import matplotlib.pyplot as plt
                plt.plot(np.fft.rfftfreq(len(trace_old), orig_binning), np.abs(fft.time2freq(trace_old, 1/orig_binning)))
                plt.plot(np.fft.rfftfreq(len(trace2), orig_binning), np.abs(fft.time2freq(trace2, 1/orig_binning)))
                plt.show()

                tt = np.arange(0, len(trace_old) * orig_binning, orig_binning)
                plt.plot(tt, trace_old, '.-')
                tt = np.arange(0, len(trace2) * orig_binning, orig_binning)
                plt.plot(tt, trace2, '.-')
                tt = np.arange(0, len(trace) / sampling_rate, 1. / sampling_rate)
                plt.plot(tt, trace, 'o-')
                plt.show()

            channel.set_trace(trace, 1. / target_binning)

    def end(self):
        pass



import scipy.optimize as opt
from scipy import signal, fftpack
import matplotlib.pyplot as plt
import numpy as np
import logging

from NuRadioReco.utilities import geometryUtilities as geo_utl
from NuRadioReco.utilities import units
from NuRadioReco.framework.parameters import stationParameters as stnp
from NuRadioReco.framework.parameters import electricFieldParameters as efp
from NuRadioReco.modules.base.module import register_run

from radiotools import helper as hp


class correlationDirectionFitter:
    """
    Fits the direction using correlation of parallel channels.
    """

    def __init__(self):
        self.__zenith = []
        self.__azimuth = []
        self.__delta_zenith = []
        self.__delta_azimuth = []
        self.logger = logging.getLogger('NuRadioReco.correlationDirectionFitter')
        self.begin()

    def begin(self, debug=False, log_level=None):
        if(log_level is not None):
            self.logger.setLevel(log_level)
        self.__debug = debug

    @register_run()
    def run(self, evt, station, det, n_index=None, ZenLim=[0 * units.deg, 90 * units.deg],
            AziLim=[0 * units.deg, 360 * units.deg],
            channel_pairs=((0, 2), (1, 3)),
            use_envelope=False):
        """
        reconstruct signal arrival direction for all events

        Parameters
        ----------
        n_index: float
            the index of refraction

        ZenLim: 2-dim array/list of floats
            the zenith angle limits for the fit
            default if 0-90deg (upward coming signal)
        AziLim: 2-dim array/list of floats
            the azimuth angle limits for the fit
            default is 0-360deg
        channel_pairs: pair of pair of integers
            specify the two channel pairs to use, default ((0, 2), (1, 3))
        use_envelope: bool (default False)
            if True, the hilbert envelope of the traces is used
        """

        use_correlation = True

        def ll_regular_station(angles, corr_02, corr_13, sampling_rate, positions, trace_start_times):
            """
            Likelihood function for a four antenna ARIANNA station, using correction.
            Using correlation, has no built in wrap around, pulse needs to be in the middle
            """

            zenith = angles[0]
            azimuth = angles[1]
            times = []

            for pos in positions:
                tmp = []
                tmp.append(geo_utl.get_time_delay_from_direction(zenith, azimuth, pos[0], n=n_index))
                tmp.append(geo_utl.get_time_delay_from_direction(zenith, azimuth, pos[1], n=n_index))
                times.append(tmp)

            delta_t_02 = times[0][1] - times[0][0]
            delta_t_13 = times[1][1] - times[1][0]
            # take different trace start times into account
            delta_t_02 -= (trace_start_times[0][1] - trace_start_times[0][0])
            delta_t_13 -= (trace_start_times[1][1] - trace_start_times[1][0])
            delta_t_02 *= sampling_rate
            delta_t_13 *= sampling_rate
            pos_02 = int(corr_02.shape[0] / 2 - delta_t_02)
            pos_13 = int(corr_13.shape[0] / 2 - delta_t_13)

#             weight_02 = np.sum(corr_02 ** 2)  # Normalize crosscorrelation
#             weight_13 = np.sum(corr_13 ** 2)
#
#             likelihood = -1 * (corr_02[pos_02] ** 2 / weight_02 + corr_13[pos_13] ** 2 / weight_13)
            # After deliberating a bit, I don't think we should use the square because anti-correlating
            # pulses would be wrong, given that it is not a continous waveform

            weight_02 = np.sum(np.abs(corr_02))  # Normalize crosscorrelation
            weight_13 = np.sum(np.abs(corr_13))

            likelihood = -1 * (corr_02[pos_02] / weight_02 + corr_13[pos_13] / weight_13)

            return likelihood

        def ll_regular_station_fft(angles, corr_02_fft, corr_13_fft, sampling_rate, positions, trace_start_times):
            """
            Likelihood function for a four antenna ARIANNA station, using FFT convolution
            Using FFT convolution, has built-in wrap around, but ARIANNA signals are too short for it to be accurate
            will show problems at zero time delay
            """

            zenith = angles[0]
            azimuth = angles[1]
            times = []

            for pos in positions:
                tmp = []
                tmp.append(geo_utl.get_time_delay_from_direction(zenith, azimuth, pos[0], n=n_index) * sampling_rate)
                tmp.append(geo_utl.get_time_delay_from_direction(zenith, azimuth, pos[1], n=n_index) * sampling_rate)
                times.append(tmp)

            delta_t_02 = (times[0][1] + trace_start_times[0][1] * sampling_rate) - (times[0][0] + trace_start_times[0][0] * sampling_rate)
            delta_t_13 = (times[1][1] + trace_start_times[1][1] * sampling_rate) - (times[1][0] + trace_start_times[1][0] * sampling_rate)

            if delta_t_02 < 0:
                pos_02 = int(delta_t_02 + corr_02_fft.shape[0])
            else:
                pos_02 = int(delta_t_02)

            if delta_t_13 < 0:
                pos_13 = int(delta_t_13 + corr_13_fft.shape[0])
            else:
                pos_13 = int(delta_t_13)

            weight_02 = np.sum(np.abs(corr_02_fft))  # Normalize crosscorrelation
            weight_13 = np.sum(np.abs(corr_13_fft))

            likelihood = -1 * (np.abs(corr_02_fft[pos_02]) ** 2 / weight_02 + np.abs(corr_13[pos_13]) ** 2 / weight_13)
            return likelihood

        station_id = station.get_id()
        positions = det.get_relative_positions(station_id)
        positions_pairs = [[positions[channel_pairs[0][0]], positions[channel_pairs[0][1]]],
                           [positions[channel_pairs[1][0]], positions[channel_pairs[1][1]]]]
        sampling_rate = station.get_channel(0).get_sampling_rate()  # assume that channels have the same sampling rate
        trace_start_time_pairs = [[station.get_channel(channel_pairs[0][0]).get_trace_start_time(), station.get_channel(channel_pairs[0][1]).get_trace_start_time()],
                                    [station.get_channel(channel_pairs[1][0]).get_trace_start_time(), station.get_channel(channel_pairs[1][1]).get_trace_start_time()]]
        # determine automatically if one channel has an inverted waveform with respect to the other
        signs = [1., 1.]
        for iPair, pair in enumerate(channel_pairs):
            antenna_type = det.get_antenna_type(station_id, pair[0])
            if("LPDA" in antenna_type):
                otheta, ophi, rot_theta, rot_azimuth = det.get_antenna_orientation(station_id, pair[0])
                otheta2, ophi2, rot_theta2, rot_azimuth2 = det.get_antenna_orientation(station_id, pair[1])
                if(np.isclose(np.abs(rot_azimuth - rot_azimuth2), 180 * units.deg, atol=1 * units.deg)):
                    signs[iPair] = -1

        if use_correlation:
            # Correlation
            if not use_envelope:
                corr_02 = signal.correlate(station.get_channel(channel_pairs[0][0]).get_trace(),
                                           signs[0] * station.get_channel(channel_pairs[0][1]).get_trace())
                corr_13 = signal.correlate(station.get_channel(channel_pairs[1][0]).get_trace(),
                                           signs[1] * station.get_channel(channel_pairs[1][1]).get_trace())
            else:
                corr_02 = signal.correlate(np.abs(signal.hilbert(station.get_channel(channel_pairs[0][0]).get_trace())),
                                           np.abs(signal.hilbert(station.get_channel(channel_pairs[0][1]).get_trace())))
                corr_13 = signal.correlate(np.abs(signal.hilbert(station.get_channel(channel_pairs[1][0]).get_trace())),
                                           np.abs(signal.hilbert(station.get_channel(channel_pairs[1][1]).get_trace())))

        else:
            # FFT convolution
            corr_02_fft = fftpack.ifft(-1 * fftpack.fft(station.get_channel(channel_pairs[0][0]).get_trace()).conjugate() * fftpack.fft(station.get_channel(channel_pairs[0][1]).get_trace()))
            corr_13_fft = fftpack.ifft(-1 * fftpack.fft(station.get_channel(channel_pairs[1][0]).get_trace()).conjugate() * fftpack.fft(station.get_channel(channel_pairs[1][1]).get_trace()))

        # Alternative fitters:
#             ll = opt.minimize(ll_regular_station,(zenith_orig-np.deg2rad(15) ,azimuth_orig-np.deg2rad(25)),
#             (corr_02, corr_13,sampling_rate),method='BFGS') # gets stuck in local minima
#
#             ll = opt.basinhopping(ll_regular_station,(zenith_orig-np.deg2rad(15) ,azimuth_orig-np.deg2rad(25)),
#                 minimizer_kwargs={'args':(corr_02, corr_13,sampling_rate)}) # does not accept range
#

        if use_correlation:
        # Using correlation
            ll = opt.brute(ll_regular_station, ranges=(slice(ZenLim[0], ZenLim[1], 0.01),
                                                       slice(AziLim[0], AziLim[1], 0.01)),
                            args=(corr_02, corr_13, sampling_rate, positions_pairs, trace_start_time_pairs),
                            full_output=True, finish=opt.fmin)  # slow but does the trick
#             print(ll)
        else:
            ll = opt.brute(ll_regular_station_fft, ranges=(slice(ZenLim[0], ZenLim[1], 0.05),
                                                           slice(AziLim[0], AziLim[1], 0.05)),
                           args=(corr_02_fft, corr_13_fft, sampling_rate, positions_pairs, trace_start_time_pairs), full_output=True, finish=opt.fmin)  # slow but does the trick

        if self.__debug:
            import peakutils
            zenith = ll[0][0]
            azimuth = ll[0][1]
            times = []

            for pos in positions_pairs:
                tmp = []
                tmp.append(geo_utl.get_time_delay_from_direction(zenith, azimuth, pos[0], n=n_index))
                tmp.append(geo_utl.get_time_delay_from_direction(zenith, azimuth, pos[1], n=n_index))
                times.append(tmp)

            delta_t_02 = times[0][1] - times[0][0]
            delta_t_13 = times[1][1] - times[1][0]
            # take different trace start times into account
            delta_t_02 -= (trace_start_time_pairs[0][1] - trace_start_time_pairs[0][0])
            delta_t_13 -= (trace_start_time_pairs[1][1] - trace_start_time_pairs[1][0])
            delta_t_02 *= sampling_rate
            delta_t_13 *= sampling_rate

            toffset = -(np.arange(0, corr_02.shape[0]) - corr_02.shape[0] / 2) / sampling_rate

            fig, (ax, ax2) = plt.subplots(2, 1, sharex=True)
            ax.plot(toffset, corr_02)
            ax.axvline(delta_t_02 / sampling_rate, label='time', c='k')
            indices = peakutils.indexes(corr_02, thres=0.8, min_dist=5)
            t02s = toffset[indices]
            ax.plot(toffset[indices], corr_02[indices], 'o')
            imax = np.argmax(corr_02[indices])
            self.logger.debug("offset 02= {:.3f}".format(toffset[indices[imax]] -  (delta_t_02 / sampling_rate)))

            ax2.plot(toffset, corr_13)
            indices = peakutils.indexes(corr_13, thres=0.8, min_dist=5)
            ax2.plot(toffset[indices], corr_13[indices], 'o')
            imax = np.argmax(corr_13[indices])
#             print("offset 13= {:.3f}".format(toffset[indices[imax]] -  (delta_t_13 / sampling_rate)))

            ax2.axvline(delta_t_13 / sampling_rate, label='time', c='k')

            ax2.set_xlabel("time")
            ax2.set_ylabel("Correlation Ch 1/ Ch3", fontsize='small')
            ax.set_ylabel("Correlation Ch 0/ Ch2", fontsize='small')
            plt.tight_layout()
#             plt.close("all")

        station[stnp.zenith] = max(ZenLim[0], min(ZenLim[1], ll[0][0]))
        station[stnp.azimuth] = ll[0][1]
        output_str = "reconstucted angles theta = {:.1f}, phi = {:.1f}".format(station[stnp.zenith] / units.deg, station[stnp.azimuth] / units.deg)
        if station.has_sim_station():
            sim_zen = None
            sim_az = None
            if(station.get_sim_station().is_cosmic_ray()):
                sim_zen = station.get_sim_station()[stnp.zenith]
                sim_az = station.get_sim_station()[stnp.azimuth]
            elif(station.get_sim_station().is_neutrino()):  # in case of a neutrino simulation, each channel has a slightly different arrival direction -> compute the average
                sim_zen = []
                sim_az = []
                for efield in station.get_sim_station().get_electric_fields_for_channels(ray_path_type='direct'):
                    sim_zen.append(efield[efp.zenith])
                    sim_az.append(efield[efp.azimuth])
                sim_zen = np.array(sim_zen)
                sim_az = hp.get_normalized_angle(np.array(sim_az))
                ops = "average incident zenith {:.1f} +- {:.1f}".format(np.mean(sim_zen) / units.deg, np.std(sim_zen) / units.deg)
                ops += " (individual: "
                for x in sim_zen:
                    ops += "{:.1f}, ".format(x / units.deg)
                ops += ")"
                self.logger.debug(ops)
                ops = "average incident azimuth {:.1f} +- {:.1f}".format(np.mean(sim_az) / units.deg, np.std(sim_az) / units.deg)
                ops += " (individual: "
                for x in sim_az:
                    ops += "{:.1f}, ".format(x / units.deg)
                ops += ")"

                self.logger.debug(ops)
                sim_zen = np.mean(np.array(sim_zen))
                sim_az = np.mean(np.array(sim_az))

            if(sim_zen is not None):
                dOmega = hp.get_angle(hp.spherical_to_cartesian(sim_zen, sim_az), hp.spherical_to_cartesian(station[stnp.zenith], station[stnp.azimuth]))
                output_str += "  MC theta = {:.2f}, phi = {:.2f},  dOmega = {:.2f}, dZen = {:.1f}, dAz = {:.1f}".format(sim_zen / units.deg, hp.get_normalized_angle(sim_az) / units.deg, dOmega / units.deg, (station[stnp.zenith] - sim_zen) / units.deg, (station[stnp.azimuth] - hp.get_normalized_angle(sim_az)) / units.deg)
                self.__zenith.append(sim_zen)
                self.__azimuth.append(sim_az)
                self.__delta_zenith.append(station[stnp.zenith] - sim_zen)
                self.__delta_azimuth.append(station[stnp.azimuth] - hp.get_normalized_angle(sim_az))

        self.logger.info(output_str)
        # Still have to add fit quality parameter to output

        if self.__debug:
            import peakutils
            # access simulated efield and high level parameters
            sim_present = False
            if(station.has_sim_station()):
                if(station.get_sim_station().has_parameter(stnp.zenith)):
                    sim_station = station.get_sim_station()
                    azimuth_orig = sim_station[stnp.azimuth]
                    zenith_orig = sim_station[stnp.zenith]
                    sim_present = True

            if sim_present:
                self.logger.debug("True CoREAS zenith {0}, azimuth {1}".format(zenith_orig, azimuth_orig))
            self.logger.debug("Result of direction fitting: [zenith, azimuth] {}".format(np.rad2deg(ll[0])))

            # Show fit space
            zen = np.arange(ZenLim[0], ZenLim[1], 1 * units.deg)
            az = np.arange(AziLim[0], AziLim[1], 2 * units.deg)

            x_plot = np.zeros(zen.shape[0] * az.shape[0])
            y_plot = np.zeros(zen.shape[0] * az.shape[0])
            z_plot = np.zeros(zen.shape[0] * az.shape[0])
            i = 0
            for a in az:
                for z in zen:
                    # Evaluate fit function for grid
                    if use_correlation:
                        z_plot[i] = ll_regular_station([z, a], corr_02, corr_13, sampling_rate, positions_pairs, trace_start_time_pairs)
                    else:
                        z_plot[i] = ll_regular_station_fft([z, a], corr_02_fft, corr_13_fft, sampling_rate, positions_pairs, trace_start_time_pairs)
                    x_plot[i] = a
                    y_plot[i] = z
                    i += 1

            fig, ax = plt.subplots(1, 1)
            ax.scatter(np.rad2deg(x_plot), np.rad2deg(y_plot), c=z_plot, cmap='gnuplot2_r', lw=0)
#             ax.imshow(z_plot, cmap='gnuplot2_r', extent=(0, 360, 90, 180))
            if sim_present:
                ax.plot(np.rad2deg(hp.get_normalized_angle(azimuth_orig)), np.rad2deg(zenith_orig), marker='d', c='g', label="True")
            ax.scatter(np.rad2deg(ll[0][1]), np.rad2deg(ll[0][0]), marker='o', c='k', label='Fit')
#             ax.colorbar(label='Fit parameter')
            ax.set_ylabel('Zenith [rad]')
            ax.set_xlabel('Azimuth [rad]')
            plt.tight_layout()

            # plot allowed solution separately for each pair of channels
            toffset = -(np.arange(0, corr_02.shape[0]) - corr_02.shape[0] / 2.) / sampling_rate
            indices = peakutils.indexes(corr_02, thres=0.8, min_dist=5)
            t02s = toffset[indices][np.argsort(corr_02[indices])[::-1]] + (trace_start_time_pairs[0][1] - trace_start_time_pairs[0][0])
            toffset = -(np.arange(0, corr_13.shape[0]) - corr_13.shape[0] / 2.) / sampling_rate
            indices = peakutils.indexes(corr_13, thres=0.8, min_dist=5)
            t13s = toffset[indices][np.argsort(corr_13[indices])[::-1]] + (trace_start_time_pairs[1][1] - trace_start_time_pairs[1][0])
            from scipy import constants
            c = constants.c * units.m / units.s
            dx = -6 * units.m

            def get_deltat13(dt, phi):
                t = -1. * dt * c / (dx * np.cos(phi) * n_index)
                t[t < 0] = np.nan
                return np.arcsin(t)

            def get_deltat02(dt, phi):
                t = -1 * dt * c / (dx * np.sin(phi) * n_index)
                t[t < 0] = np.nan
                return np.arcsin(t)

            def getDeltaTCone(r, dt):
                dist = np.linalg.norm(r)
                t0 = -dist * n_index / c
                Phic = np.arccos(dt / t0)  # cone angle for allowable solutions
                self.logger.debug('dist = {}, dt = {}, t0 = {}, phic = {}'.format(dist, dt, t0, Phic))
                nr = r / dist  # normalize
                p = np.cross([0, 0, 1], nr)  # create a perpendicular normal vector to r
                p = p / np.linalg.norm(p)
                q = np.cross(nr, p)  # nr, p, and q form an orthonormal basis
                self.logger.debug('nr = {}\np = {}\nq = {}\n'.format(nr, p, q))
                ThetaC = np.linspace(0, 2 * np.pi, 1000)
                Phis = np.zeros(len(ThetaC))
                Thetas = np.zeros(len(ThetaC))
                for i, thetac in enumerate(ThetaC):
                    # create a set of vectors that point along the cone defined by r and PhiC
                    rc = nr + np.tan(Phic) * (np.sin(thetac) * p + np.cos(thetac) * q)
                    nrc = rc / np.linalg.norm(rc)
                    theta = np.arccos(nrc[2])
                    phi = np.arctan2(nrc[1], nrc[0])
                    Phis[i] = phi
                    Thetas[i] = theta
                return Phis, Thetas

            # phis = np.deg2rad(np.linspace(0, 360, 10000))
            r0_2 = positions_pairs[0][1] - positions_pairs[0][0]  # vector pointing from Ch2 to Ch0
            r1_3 = positions_pairs[1][1] - positions_pairs[1][0]  # vector pointing from Ch3 to Ch1
            self.logger.debug('r02 {}\nr13 {}'.format(r0_2, r1_3))
            linestyles = ['-', '--', ':', '-.']
            for i, t02 in enumerate(t02s):
                # theta02 = get_deltat02(t02, phis)
                phi02, theta02 = getDeltaTCone(r0_2, t02)
                theta02[theta02 < 0] += np.pi
                phi02[phi02 < 0] += 2 * np.pi
                jumppos02 = np.where(np.abs(np.diff(phi02)) >= 5.0)
                for j, pos in enumerate(jumppos02):
                    phi02 = np.insert(phi02, pos + 1 + j, np.nan)
                    theta02 = np.insert(theta02, pos + 1 + j, np.nan)
                # mask02 = ~np.isnan(theta02)
                ax.plot(np.rad2deg(phi02), np.rad2deg(theta02), '{}C3'.format(linestyles[i % 4]), label='c 0+2 dt = {}'.format(t02))
            for i, t13 in enumerate(t13s):
                # theta13 = get_deltat13(t13, phis)
                phi13, theta13 = getDeltaTCone(r1_3, t13)
                theta13[theta13 < 0] += np.pi
                phi13[phi13 < 0] += 2 * np.pi
                jumppos13 = np.where(np.abs(np.diff(phi13)) >= 5.0)
                for j, pos in enumerate(jumppos13):
                    phi13 = np.insert(phi13, pos + 1 + j, np.nan)
                    theta13 = np.insert(theta13, pos + 1 + j, np.nan)
                # mask13 = ~np.isnan(theta13)
                ax.plot(np.rad2deg(phi13), np.rad2deg(theta13), '{}C2'.format(linestyles[i % 4]), label='c 1+3 dt = {}'.format(t13))
            ax.legend(fontsize='small')
            ax.set_ylim(ZenLim[0] / units.deg, ZenLim[1] / units.deg)
            ax.set_xlim(AziLim[0] / units.deg, AziLim[1] / units.deg)

            # plot expectation
            # import expectation as e
            # zenith_expected = np.pi - e.get_arrival_angle(time.mktime(station.get_station_time().timetuple()))
            # ax.plot(225.5, np.rad2deg(zenith_expected), 'xr', label='expectation')

#             plt.legend()

    def end(self):
        fig, ax = plt.subplots(1, 1)
        mask = np.abs(self.__delta_azimuth) < (1 * units.deg)
        ax.scatter(np.array(self.__zenith)[mask] / units.deg, np.array(self.__delta_zenith)[mask] / units.deg, s=20)
        ax.set_xlabel("zenith angle (MC) [deg]")
        ax.set_ylabel("(zenith_rec - zenith_MC) [deg]")
        fig.tight_layout()
        fig.savefig("zenith_bias.png")

        from radiotools import plthelpers as php
        bins = np.arange(-10, 10, .1)
        fig, ax = php.get_histogram(np.array(self.__delta_azimuth) / units.deg, bins=bins, xlabel="delta azimuth [deg]")
        fig.savefig("azimuth.png")
        plt.show()
        pass



import dash
import json
import plotly
from NuRadioReco.utilities import units, templates, fft
from NuRadioReco.eventbrowser.default_layout import default_layout
from NuRadioReco.framework.parameters import stationParameters as stnp
from NuRadioReco.utilities import geometryUtilities
from NuRadioReco.utilities import trace_utilities
import numpy as np
import dash_html_components as html
import dash_core_components as dcc
from dash.dependencies import Input, Output, State
from app import app
import os
import NuRadioReco.detector.antennapattern
import dataprovider
provider = dataprovider.DataProvider()
if 'NURADIORECOTEMPLATES' in os.environ:
    template_provider = templates.Templates(os.environ.get('NURADIORECOTEMPLATES'))
else:
    template_provider = templates.Templates('')
antenna_pattern_provider = NuRadioReco.detector.antennapattern.AntennaPatternProvider()

layout = [
    html.Div([
        html.Div([
            dcc.Dropdown(id='dropdown-traces',
                options=[
                    {'label': 'calibrated trace', 'value': 'trace'},
                    {'label': 'cosmic-ray template', 'value': 'crtemplate'},
                    {'label': 'neutrino template', 'value': 'nutemplate'},
                    {'label': 'envelope', 'value': 'envelope'},
                    {'label': 'from rec. E-field', 'value': 'efield'}
                ],
                multi=True,
                value=["trace"]
            )
        ], style={'flex': '1'}),
        html.Div([
            dcc.Dropdown(id='dropdown-trace-info',
                options=[
                    {'label': 'RMS', 'value': 'RMS'},
                    {'label': 'L1', 'value': 'L1'}
                ],
                multi=True,
                value=["RMS", "L1"]
            )
        ], style={'flex': '1'}),
    ], style={'display': 'flex'}),
    html.Div([
        html.Div([
            html.Div([
                html.Div('Template directory', className='input-group-addon'),
                dcc.Input(id='template-directory-input', placeholder='template directory', className='form-control'),
                html.Div([
                    html.Button('load', id='open-template-button', className='btn btn-default')
                ], className='input-group-btn')
            ], className='input-group', id='template-input-group')
        ], style={'flex': '1'}),
        html.Div('', style={'flex': '1'})
    ], style={'display': 'flex'}),
    dcc.Graph(id='time-traces')
]

@app.callback(
    dash.dependencies.Output('dropdown-traces', 'options'),
    [dash.dependencies.Input('event-counter-slider', 'value'),
     dash.dependencies.Input('filename', 'value'),
     dash.dependencies.Input('station-id-dropdown', 'value')],
     [State('user_id', 'children')]
)
def get_dropdown_traces_options(evt_counter, filename, station_id, juser_id):
    if filename is None or station_id is None:
        return []
    user_id = json.loads(juser_id)
    ariio = provider.get_arianna_io(user_id, filename)
    evt = ariio.get_event_i(evt_counter)
    station = evt.get_station(station_id)
    options=[
        {'label': 'calibrated trace', 'value': 'trace'},
        {'label': 'cosmic-ray template', 'value': 'crtemplate'},
        {'label': 'neutrino template', 'value': 'nutemplate'},
        {'label': 'envelope', 'value': 'envelope'},
        {'label': 'from rec. E-field', 'value': 'recefield'}
    ]
    if station.get_sim_station() is not None:
        if len(station.get_sim_station().get_electric_fields()) > 0:
            options.append({'label': 'from sim. E-field', 'value': 'simefield'})
    return options

@app.callback(
    Output('template-input-group', 'style'),
    [Input('dropdown-traces', 'value')]
)
def show_template_input(trace_dropdown_options):
    if 'NURADIORECOTEMPLATES' in os.environ:
        return {'display': 'none'}
    if 'crtemplate' in trace_dropdown_options or 'nutemplate' in trace_dropdown_options:
        return {}
    return {'display': 'none'}

def get_L1(a):
    ct = np.array(a[1:]) ** 2
    l1 = np.max(ct) / (np.sum(ct) - np.max(ct))
    return l1

@app.callback(
    dash.dependencies.Output('time-traces', 'figure'),
    [dash.dependencies.Input('event-counter-slider', 'value'),
     dash.dependencies.Input('filename', 'value'),
     dash.dependencies.Input('dropdown-traces', 'value'),
     dash.dependencies.Input('dropdown-trace-info', 'value'),
     dash.dependencies.Input('station-id-dropdown', 'value'),
     dash.dependencies.Input('open-template-button', 'n_clicks_timestamp')],
     [State('user_id', 'children'),
     State('template-directory-input', 'value')])
def update_multi_channel_plot(evt_counter, filename, dropdown_traces, dropdown_info, station_id, open_template_timestamp, juser_id, template_directory):
    if filename is None or station_id is None:
        return {}
    user_id = json.loads(juser_id)
    colors = plotly.colors.DEFAULT_PLOTLY_COLORS
    ariio = provider.get_arianna_io(user_id, filename)
    evt = ariio.get_event_i(evt_counter)
    station = evt.get_station(station_id)
    traces = []
    ymax = 0
    n_channels = 0
    plot_titles = []
    trace_start_times = []
    fig = plotly.subplots.make_subplots(rows=station.get_number_of_channels(), cols=2,
        shared_xaxes=True, shared_yaxes=False,
        vertical_spacing=0.01, subplot_titles=plot_titles)
    for i, channel in enumerate(station.iter_channels()):
        n_channels += 1
        trace = channel.get_trace() / units.mV
        ymax = max(ymax, np.max(np.abs(trace)))
        plot_titles.append('Channel {}'.format(channel.get_id()))
        plot_titles.append('Channel {}'.format(channel.get_id()))
        trace_start_times.append(channel.get_trace_start_time())
        if channel.get_trace() is not None:
            trace = channel.get_trace() / units.mV
            ymax = max(ymax, np.max(np.abs(trace)))
    if np.min(trace_start_times) > 1000.*units.ns:
        trace_start_time_offset = np.floor(np.min(trace_start_times)/1000.)*1000.
    else:
        trace_start_time_offset = 0
    if 'trace' in dropdown_traces:
        for i, channel in enumerate(station.iter_channels()):
            tt = channel.get_times() - trace_start_time_offset / units.ns
            if channel.get_trace() is None:
                continue
            trace = channel.get_trace() / units.mV
            fig.append_trace(plotly.graph_objs.Scatter(
                    x=tt,
                    y=trace,
                    # text=df_by_continent['country'],
                    # mode='markers',
                    opacity=0.7,
                    marker={
                        'color': colors[i % len(colors)],
                        'line': {'color': colors[i % len(colors)]}
                    },
                    name=i
                ), i + 1, 1)
            if 'RMS' in dropdown_info:
                fig.append_trace(
                   plotly.graph_objs.Scatter(
                        x=[0.99 * tt.max()],
                        y=[0.98 * trace.max()],
                        mode='text',
                        text=[r'mu = {:.2f}, STD={:.2f}'.format(np.mean(trace), np.std(trace))],
                        textposition='bottom left'
                    ),i + 1, 1)
    if 'envelope' in dropdown_traces:
        for i, channel in enumerate(station.iter_channels()):
            if channel.get_trace() is None:
                continue
            trace = channel.get_trace() / units.mV
            from scipy import signal
            yy = np.abs(signal.hilbert(trace))
            fig.append_trace(plotly.graph_objs.Scatter(
                    x=channel.get_times() - trace_start_time_offset / units.ns,
                    y=yy,
                    # text=df_by_continent['country'],
                    # mode='markers',
                    opacity=0.7,
                    line=dict(
                        width=4,
                        dash='dot'),  # dash options include 'dash', 'dot', and 'dashdot'
                    marker={
                        'color': colors[i % len(colors)],
                        'line': {'color': colors[i % len(colors)]}
                    },
                    name=i
                ), i + 1, 1)
    if 'crtemplate' in dropdown_traces:
        if not 'NURADIORECOTEMPLATES' in os.environ: # if the environment variable is not set, we have to ask the user to specify the template location
            template_provider.set_template_directory(template_directory)
        ref_template = template_provider.get_cr_ref_template(station_id)
        if(station.has_parameter(stnp.cr_xcorrelations)):
            ref_templates = template_provider.get_set_of_cr_templates(station_id, n=station.get_parameter(stnp.cr_xcorrelations)['number_of_templates'])
        for i, channel in enumerate(station.iter_channels()):
            if(channel.has_parameter(chp.cr_xcorrelations)):
                key = channel.get_parameter(chp.cr_xcorrelations)['cr_ref_xcorr_template']
                logger.info("using template {}".format(key))
                ref_template = ref_templates[key][channel.get_id()]
            times = channel.get_times() - trace_start_time_offset
            trace = channel.get_trace()
            if trace is None:
                continue
            xcorr = channel.get_parameter(chp.cr_xcorrelations)['cr_ref_xcorr']
            xcorrpos = channel.get_parameter(chp.cr_xcorrelations)['cr_ref_xcorr_time']
            dt = times[1] - times[0]
            xcorr_max = xcorr
            if(channel.has_parameter(chp.cr_xcorrelations)):
                xcorr_max = channel.get_parameter(chp.cr_xcorrelations)['cr_ref_xcorr_max']
            flip = np.sign(xcorr_max)
            tttemp = np.arange(0, len(ref_template) * dt, dt)
            yy = flip * ref_template * np.abs(trace).max()
            fig.append_trace(plotly.graph_objs.Scatter(
                    x=tttemp[:len(trace)] / units.ns,
                    y=yy[:len(trace)] / units.mV,
                    # text=df_by_continent['country'],
                    # mode='markers',
                    opacity=0.7,
                    line=dict(
                        width=2,
                        dash='dot'),  # dash options include 'dash', 'dot', and 'dashdot'
                    marker={
                        'color': colors[i % len(colors)],
                        'line': {'color': colors[i % len(colors)]}
                    },
                    name=i
                ), i + 1, 1)
            template_spectrum = fft.time2freq(yy, channel.get_sampling_rate())
            template_freqs = np.fft.rfftfreq(len(yy), dt)
            template_freq_mask = (template_freqs > channel.get_frequencies()[0])&(template_freqs<(channel.get_frequencies()[-1]))
            fig.append_trace(plotly.graph_objs.Scatter(
                x = template_freqs[template_freq_mask] / units.MHz,
                y = np.abs(template_spectrum)[template_freq_mask] / units.mV,
                line=dict(
                    width=2,
                    dash='dot'
                ),
                name=i
            ), i+1, 2)
            fig.append_trace(
               plotly.graph_objs.Scatter(
                    x=[0.9 * times.max() / units.ns],
                    y=[0.8 * trace.max() / units.mV],
                    mode='text',
                    text=['cr xcorr= {:.2f}, {:.02f}'.format(xcorr, xcorr_max)],
                    textposition='top center'
                ),
            i + 1, 1)
            fig.append_trace(
               plotly.graph_objs.Scatter(
                    x=[0.9 * times.max() / units.ns],
                    y=[0.5 * trace.max() / units.mV],
                    mode='text',
                    text=['t = {:.0f}ns'.format(xcorrpos)],
                    textposition='top center'
                ),
            i + 1, 1)
    if 'nutemplate' in dropdown_traces:
        if not 'NURADIORECOTEMPLATES' in os.environ: # if the environment variable is not set, we have to ask the user to specify the template location
            template_provider.set_template_directory(template_directory)
        ref_template = template_provider.get_nu_ref_template(station_id)
        for i, channel in enumerate(station.iter_channels()):
            times = channel.get_times()
            trace = channel.get_trace()
            if trace is None:
                continue
            xcorr = channel.get_parameter(chp.nu_xcorrelations)['nu_ref_xcorr']
            xcorrpos = channel.get_parameter(chp.nu_xcorrelations)['nu_ref_xcorr_time']
            dt = times[1] - times[0]
            flip = np.sign(xcorr)
            tttemp = np.arange(0, len(ref_template) * dt, dt)
            yy = flip * ref_template * np.abs(trace).max()
            fig.append_trace(plotly.graph_objs.Scatter(
                    x=tttemp[:len(trace)] / units.ns,
                    y=yy[:len(trace)] / units.mV,
                    # text=df_by_continent['country'],
                    # mode='markers',
                    opacity=0.7,
                    line=dict(
                        width=4,
                        dash='dot'),  # dash options include 'dash', 'dot', and 'dashdot'
                    marker={
                        'color': colors[i % len(colors)],
                        'line': {'color': colors[i % len(colors)]}
                    },
                    name=i
                ), i + 1, 1)
            template_spectrum = fft.time2freq(yy, channel.get_sampling_rate())
            template_freqs = np.fft.rfftfreq(len(yy), dt)
            template_freq_mask = (template_freqs > channel.get_frequencies()[0])&(template_freqs<(channel.get_frequencies()[-1]))
            fig.append_trace(plotly.graph_objs.Scatter(
                x = template_freqs[template_freq_mask] / units.MHz,
                y = np.abs(template_spectrum)[template_freq_mask] / units.mV,
                line=dict(
                    width=2,
                    dash='dot'
                ),
                name=i
            ), i+1, 2)

            fig.append_trace(
               plotly.graph_objs.Scatter(
                    x=[0.9 * times.max() / units.ns],
                    y=[0.8 * trace.max() / units.mV],
                    mode='text',
                    text=['nu xcorr= {:.2f}'.format(xcorr)],
                    textposition='top center'
                ),
            i + 1, 1)
            fig.append_trace(
               plotly.graph_objs.Scatter(
                    x=[0.9 * times.max() / units.ns],
                    y=[0.5 * trace.max() / units.mV],
                    mode='text',
                    text=['t = {:.0f}ns'.format(xcorrpos)],
                    textposition='top center'
                ),
            i + 1, 1)
    if 'recefield' in dropdown_traces or 'simefield' in dropdown_traces:
        det.update(station.get_station_time())
    if 'recefield' in dropdown_traces:
        channel_ids = []
        for channel in station.iter_channels():
            channel_ids.append(channel.get_id())
        for electric_field in station.get_electric_fields():
            for i_trace, trace in enumerate(trace_utilities.get_channel_voltage_from_efield(station, electric_field, channel_ids, det, station.get_parameter(stnp.zenith), station.get_parameter(stnp.azimuth), antenna_pattern_provider)):
                    channel = station.get_channel(channel_ids[i_trace])
                    direction_time_delay = geometryUtilities.get_time_delay_from_direction(station.get_parameter(stnp.zenith), station.get_parameter(stnp.azimuth), det.get_relative_position(station.get_id(),channel_ids[i_trace]) - electric_field.get_position())
                    time_shift = direction_time_delay - trace_start_time_offset
                    plotly.graph_objs.append_trace(plotly.graph_objs.Scatter(
                        x=(electric_field.get_times() + time_shift)/units.ns,
                        y=fft.freq2time(trace, electric_field.get_sampling_rate())/units.mV,
                        line=dict(
                            dash='solid',
                            color=colors[i_trace%len(colors)]
                        ),
                        opacity=.5
                    ), i_trace+1, 1)
                    plotly.graph_objs.append_trace(plotly.graph_objs.Scatter(
                        x=electric_field.get_frequencies()/units.MHz,
                        y=np.abs(trace)/units.mV,
                        line=dict(
                            dash='solid',
                            color=colors[i_trace%len(colors)]
                        ),
                        opacity=.5
                    ), i_trace + 1, 2)
    if 'simefield' in dropdown_traces:
        channel_ids = []
        sim_station = station.get_sim_station()
        for i_channel, channel in enumerate(station.iter_channels()):
            for electric_field in sim_station.get_electric_fields_for_channels([channel.get_id()]):
                trace = trace_utilities.get_channel_voltage_from_efield(sim_station, electric_field, [channel.get_id()], det, electric_field.get_parameter(efp.zenith), electric_field.get_parameter(efp.azimuth), antenna_pattern_provider)[0]
                channel = station.get_channel(channel.get_id())
                if station.is_cosmic_ray():
                    direction_time_delay = geometryUtilities.get_time_delay_from_direction(sim_station.get_parameter(stnp.zenith), sim_station.get_parameter(stnp.azimuth), det.get_relative_position(sim_station.get_id(),channel.get_id()) - electric_field.get_position())
                    time_shift = direction_time_delay - trace_start_time_offset
                else:
                    time_shift = - trace_start_time_offset
                fig.append_trace(plotly.graph_objs.Scatter(
                    x=(electric_field.get_times() + time_shift)/units.ns,
                    y=fft.freq2time(trace, electric_field.get_sampling_rate())/units.mV,
                    line=dict(
                        dash='solid',
                        color=colors[i_channel%len(colors)]
                    ),
                    opacity=.5
                ), i_channel+1, 1)
                fig.append_trace(plotly.graph_objs.Scatter(
                    x=electric_field.get_frequencies()/units.MHz,
                    y=np.abs(trace)/units.mV,
                    line=dict(
                        dash='solid',
                        color=colors[i_channel%len(colors)]
                    ),
                    opacity=.5
                ), i_channel + 1, 2)
    for i, channel in enumerate(station.iter_channels()):
        fig['layout']['yaxis{:d}'.format(i * 2 + 1)].update(range=[-ymax, ymax])
        fig['layout']['yaxis{:d}'.format(i * 2 + 1)].update(title='voltage [mV]')

        if channel.get_trace() is None:
            continue
        spec = channel.get_frequency_spectrum()
        ff = channel.get_frequencies()
        fig.append_trace(plotly.graph_objs.Scatter(
                x=ff / units.MHz,
                y=np.abs(spec) / units.mV,
                opacity=0.7,
                marker={
                    'color': colors[i % len(colors)],
                    'line': {'color': colors[i % len(colors)]}
                },
                name=i
            ), i + 1, 2)
        if 'L1' in dropdown_info:
            fig.append_trace(
                   plotly.graph_objs.Scatter(
                        x=[0.9 * ff.max() / units.MHz],
                        y=[0.8 * np.abs(spec).max() / units.mV],
                        mode='text',
                        text=['max L1 = {:.2f}'.format(get_L1(spec))],
                        textposition='top center'
                    ),
                i + 1, 2)
    if trace_start_time_offset > 0:
        fig['layout']['xaxis1'].update(title='time [ns] - {:.0f}ns'.format(trace_start_time_offset))
    else:
        fig['layout']['xaxis1'].update(title='time [ns]')
    fig['layout']['xaxis2'].update(title='frequency [MHz]')
    fig['layout'].update(height=n_channels*150)
    fig['layout'].update(showlegend=False)
    return fig



import numpy as np
from scipy.interpolate import interp1d
import os
from radiotools import helper as hp
from NuRadioReco.utilities import units, io_utilities
import logging
logger = logging.getLogger('analog_components')


def load_amplifier_response(amp_type='100', path=os.path.dirname(os.path.realpath(__file__))):
    """
    Read out amplifier gain and phase. Currently only examples have been implemented.
    Needs a better structure in the future, possibly with database.
    """
    amplifier_response = {}
    if amp_type == '100':
        amp_gain_discrete = np.loadtxt(os.path.join(path, 'HardwareResponses/Amp109_SHP100SLP1000_3L3_60dB.csv'),
                                       skiprows=44, delimiter=',')
        ph = os.path.join(path, 'HardwareResponses/AMP109_SHP100SLP1000_3L3_PHASE.CSV')
        amp_phase_discrete = np.loadtxt(ph, skiprows=3, delimiter=',')
    elif amp_type == '200':
        amp_gain_discrete = np.loadtxt(os.path.join(path, 'HardwareResponses/amp_200_logmag.csv'),
                                       skiprows=3, delimiter=',')
        ph = os.path.join(path, 'HardwareResponses/amp_200_phase.csv')
        amp_phase_discrete = np.loadtxt(ph, skiprows=3, delimiter=',')
    elif amp_type == '300':
        amp_gain_discrete = np.loadtxt(os.path.join(path, 'HardwareResponses/amp_300_gain.csv'),
                                       skiprows=3, delimiter=',')
        ph = os.path.join(path, 'HardwareResponses/amp_300_phase.csv')
        amp_phase_discrete = np.loadtxt(ph, skiprows=3, delimiter=',')
    else:
        logger.error("Amp type not recognized")
        return amplifier_response

    # Convert to GHz and add 60dB/40dB for attenuation in measurement circuit
    amp_gain_discrete[:, 0] *= units.Hz

    if amp_type == '300':
        amp_gain_discrete[:, 1] += 40
    elif amp_type == '100':
        amp_gain_discrete[:, 1] += 60
    elif amp_type == '200':
        amp_gain_discrete[:, 1] += 60

    amp_gain_db_f = interp1d(amp_gain_discrete[:, 0], amp_gain_discrete[:, 1],
                             bounds_error=False, fill_value=0)  # all requests outside of measurement range are set to 0

    def get_amp_gain(ff):
        amp_gain_db = amp_gain_db_f(ff)
        amp_gain = 10 ** (amp_gain_db / 20.)
        return amp_gain

    # Convert to MHz and broaden range
    amp_phase_discrete[:, 0] *= units.Hz

    amp_phase_f = interp1d(amp_phase_discrete[:, 0], np.unwrap(np.deg2rad(amp_phase_discrete[:, 1])),
                           bounds_error=False, fill_value=0)  # all requests outside of measurement range are set to 0

    def get_amp_phase(ff):
        amp_phase = amp_phase_f(ff)
        return np.exp(1j * amp_phase)

    amplifier_response['gain'] = get_amp_gain
    amplifier_response['phase'] = get_amp_phase

    return amplifier_response

def load_amp_measurement(amp_measurement):
    """
    load individual amp measurement from file and buffer interpolation function
    """
    filename = os.path.join(os.path.dirname(__file__), 'HardwareResponses/', amp_measurement+".pkl")
    data = io_utilities.read_pickle(filename, encoding='latin1')
    if(amp_measurement not in data):
        raise AttributeError("can't find amp measurement {}".format(amp_measurement))
    ff = data[amp_measurement]['freqs']
    response = data[amp_measurement]['response']
    gain = np.abs(response)
    phase = np.unwrap(np.angle(response))
    amp_phase_f = interp1d(ff, phase, bounds_error=False, fill_value=0)  # all requests outside of measurement range are set to 0
    amp_gain_f = interp1d(ff, gain, bounds_error=False, fill_value=1)  # all requests outside of measurement range are set to 1
    
    def get_response(ff):
        return amp_gain_f(ff) * np.exp(1j * amp_phase_f(ff))
    
    amp_measurements[amp_measurement] = get_response

# amp responses do not occupy a lot of memory, pre load all responses
amplifier_response = {}
for amp_type in ['100', '200', '300']:
    amplifier_response[amp_type] = load_amplifier_response(amp_type)
    
amp_measurements = {}  # buffer for amp measurements


def get_amplifier_response(ff, amp_type, amp_measurement=None):
    if(amp_measurement is not None):
        if amp_measurement not in amp_measurements:
            load_amp_measurement(amp_measurement)
        return amp_measurements[amp_measurement](ff)
    elif(amp_type in amplifier_response.keys()):
        return amplifier_response[amp_type]['gain'](ff) * amplifier_response[amp_type]['phase'](ff)
    else:
        logger.error("Amplifier response for type {} not implemented, returning None".format(amp_type))
        return None


def get_cable_response_parametrized(frequencies, cable_type, cable_length):
    if(cable_type == "LMR_400"):

        def attn_db_per_100ft(f):  # from LMR-400 spec sheet
            return 0.122290 * (f / units.MHz) ** 0.5 + 0.000260 * f / units.MHz   # https://www.timesmicrowave.com/DataSheets/CableProducts/LMR-400.pdf

        logger.debug("{} {} {}".format(cable_type, cable_length, type(cable_length)))
        attn = attn_db_per_100ft(frequencies) / (100 * units.feet) * cable_length
        attn += 0.01  # dB connetor loss
        return 1. / hp.dB_to_linear(attn) ** 0.5
    elif(cable_type == "LMR_240"):

        def attn_db_per_100ft(f):  # from LMR-400 spec sheet
            return 0.242080 * (f / units.MHz) ** 0.5 + 0.000330 * f / units.MHz    # https://www.timesmicrowave.com/DataSheets/CableProducts/LMR-240.pdf

        logger.debug("{} {} {}".format(cable_type, cable_length, type(cable_length)))
        attn = attn_db_per_100ft(frequencies) / (100 * units.feet) * cable_length
        attn += 0.01  # dB connetor loss
        return 1. / hp.dB_to_linear(attn) ** 0.5 
    else:
        logger.error("cable type {} not defined".format(cable_type))
        raise NotImplementedError


def get_cable_response(frequencies, path=os.path.dirname(os.path.realpath(__file__))):
    """
    Read out cable induced loss and phase. From standard 4 channel station.
    """

    cable_discrete = np.loadtxt(os.path.join(path, 'HardwareResponses/CableAntennuation_James2016.csv'), skiprows=1, delimiter=',')

    max_frequency = 5000. * units.MHz
    if np.max(frequencies) > max_frequency:
        max_frequency = np.max(frequencies)

    # Convert to GHz
    cable_discrete[:, 0] *= units.Hz
    cable_discrete[0, 0] = 0.
    cable_discrete[-1, 0] = max_frequency

    cable_amp_db_f = interp1d(cable_discrete[:, 0], cable_discrete[:, 1])
    cable_amp_db = cable_amp_db_f(frequencies)
    cable_amp = 10 ** (cable_amp_db / 20.)

    cable_phase_f = interp1d(cable_discrete[:, 0], np.unwrap(np.deg2rad(cable_discrete[:, 2])))
    cable_phase = cable_phase_f(frequencies)
    cable_phase = np.exp(1j * cable_phase)

    return cable_amp * cable_phase




import plotly.graph_objects as go
import plotly.io as pio
pio.renderers.default = "browser"
import numpy as np
import sys
from NuRadioReco.detector import generic_detector as detector
from NuRadioReco.utilities import units
from radiotools import helper as hp
from astropy.time import Time

det = detector.GenericDetector(json_filename=sys.argv[1], default_station=101)
det.update(Time.now())

sid = 101
l = 1 * units.m

data = []

for cid in det.get_channel_ids(sid):
    ori_zen, ori_az, rot_zen, rot_az = det.get_antenna_orientation(sid, cid)
    x, y, z = det.get_relative_position(sid, cid)

    dx, dy, dz = hp.spherical_to_cartesian(ori_zen, ori_az)
    xx = [x, x - l * dx]
    yy = [y, y - l * dy]
    zz = [z, z - l * dz]
    dx, dy, dz = hp.spherical_to_cartesian(rot_zen, rot_az)
    xx.insert(0, x + 0.5 * l * dx)
    yy.insert(0, y + 0.5 * l * dy)
    zz.insert(0, z + 0.5 * l * dz)
    print(f"channel {cid:d}, rot = {dx:.1f}, {dy:.1f}, {dz:.1f}, ({rot_zen/units.deg:.0f}, {rot_az/units.deg:.0f})")

    data.append(go.Scatter3d(x=xx, y=yy, z=zz, mode='lines+markers'))

    if "LPDA" in det.get_antenna_type(sid, cid):
        ori = hp.spherical_to_cartesian(ori_zen, ori_az)
        rot = hp.spherical_to_cartesian(rot_zen, rot_az)
        dx, dy, dz = hp.spherical_to_cartesian(ori_zen, ori_az)
        v = np.cross(ori, rot)
        xx = [x]
        yy = [y]
        zz = [z]
        xx.extend([x - l * dx + 0.5 * l * v[0], x - l * dx - 0.5 * l * v[0], x])
        yy.extend([y - l * dy + 0.5 * l * v[1], y - l * dy - 0.5 * l * v[1], y])
        zz.extend([z - l * dz + 0.5 * l * v[2], z - l * dz - 0.5 * l * v[2], z])

        data.append(go.Mesh3d(x=xx, y=yy, z=zz, opacity=0.5, color='blue',
                              alphahull=-1, delaunayaxis='x'))
        data.append(go.Mesh3d(x=xx, y=yy, z=zz, opacity=0.5, color='blue',
                              alphahull=-1, delaunayaxis='y'))
#         data.append(go.Scatter3d(x=xx, y=yy, z=zz, mode='markers'))

fig = go.Figure(data=data)
fig.update_layout(scene=dict(aspectmode='data', aspectratio=dict(x=1, y=1, z=2)))
fig.show()



import dash
import json
import plotly
from NuRadioReco.utilities import units
from NuRadioReco.eventbrowser.default_layout import default_layout, efield_plot_colors, efield_plot_linestyles, polarizaiton_names
from NuRadioReco.framework.parameters import electricFieldParameters as efp
import dash_html_components as html
import dash_core_components as dcc
from dash.dependencies import Input, Output, State
from app import app
import dataprovider
provider = dataprovider.DataProvider()

layout = [
    html.Div([
        html.Div([
            html.Div('Signal Type', className=''),
            dcc.Checklist(
                id='sim-traces-signal-types',
                options = [
                    {'label': 'direct', 'value': 'direct'},
                    {'label': 'reflected', 'value': 'reflected'},
                    {'label': 'refracted', 'value': 'refracted'}
                ],
                value = ['direct'],
                className='sim-trace-option'
            )
        ], className='', id='sim-traces-signal-types-container')
    ], className='sim-trace-options'),
    html.Div([
        dcc.Graph(id='sim-traces')
    ]),
]

@app.callback(
    Output('sim-traces', 'figure'),
    [Input('event-counter-slider', 'value'),
    Input('filename', 'value'),
    Input('sim-traces-signal-types', 'value'),
    Input('station-id-dropdown', 'value')],
    [State('user_id', 'children')]
)
def update_sim_trace_plot(i_event, filename, signal_types, station_id, juser_id):
    if filename is None or station_id is None:
        return {}
    user_id = json.loads(juser_id)
    colors = plotly.colors.DEFAULT_PLOTLY_COLORS
    ariio = provider.get_arianna_io(user_id, filename)
    evt = ariio.get_event_i(i_event)
    station = evt.get_station(station_id)
    sim_station = station.get_sim_station()
    visibility_settings = ['legendonly', True, True]
    if sim_station is None:
        return {}
    fig = plotly.subplots.make_subplots(rows=1, cols=1)
    for i_electric_field, electric_field in enumerate(sim_station.get_electric_fields()):
        if electric_field.get_parameter(efp.ray_path_type) in signal_types:
            for polarization in range(1,3):
                fig.append_trace(
                    plotly.graph_objs.Scatter(
                        x=electric_field.get_times()/units.ns,
                        y=electric_field.get_trace()[polarization]/units.mV*units.m,
                        opacity=1.,
                        line = {
                            'color': efield_plot_colors[i_electric_field % len(efield_plot_colors)][polarization-1],
                            'dash': efield_plot_linestyles[electric_field.get_parameter(efp.ray_path_type)]
                        },
                        name = 'Ch. {} {} ({})'.format(electric_field.get_channel_ids(), polarizaiton_names[polarization], electric_field.get_parameter(efp.ray_path_type)),
                        legendgroup=str(i_electric_field),
                        visible=visibility_settings[polarization]
                    ), 1, 1
                )
    fig['layout'].update(default_layout)
    fig['layout'].update(
        xaxis={'title': 't [ns]'},
        yaxis={'title': 'electric field [mV/m]'}
    )
    return fig



import NuRadioReco.modules.channelGenericNoiseAdder
from NuRadioReco.utilities import units
import matplotlib.pyplot as plt
import numpy as np
from scipy import constants
import argparse
channelGenericNoiseAdder = NuRadioReco.modules.channelGenericNoiseAdder.channelGenericNoiseAdder()

"""
This code calculates the probability per window for the whole phased
array (not just one beam) of triggering on noise, and the rate of
noise triggers.

The trigger is an ARA-like power trigger and a phased array. According
to the ARA folks, the maximum rate should be around 10 Hz for the whole
phased array.

This code calculates the delays for the primary and secondary beams, couples
a primary to a secondary, introduces noise in all the phased antennas and then
phases the array into the directions given by default_angles and default_sec_angles.
It verifies if there has been a trigger and repeats.

Increasing the number of samples is equivalent to increasing the number of
tries, since the code calculates the probability of randomly triggering a
single window filled with noise.
"""

parser = argparse.ArgumentParser(description='calculates noise trigger rate for phased array')
parser.add_argument('--ntries', type=int, help='number noise traces to which a trigger is applied for each threshold', default=1000)
parser.add_argument('--array', type=str, help='array type: RNO or ARA', default='RNO')
args = parser.parse_args()

main_low_angle = -53. * units.deg
main_high_angle = 47. * units.deg
default_angles = np.arcsin( np.linspace( np.sin(main_low_angle), np.sin(main_high_angle), 15) )

default_sec_angles = 0.5*(default_angles[:-1]+default_angles[1:])
default_sec_angles = np.insert(default_sec_angles, len(default_sec_angles), default_angles[-1] + 3.5*units.deg)

def get_beam_rolls(ant_z, channel_list, phasing_angles=default_angles, time_step=2./3*units.ns, ref_index=1.78):
    """
    Calculates the delays needed for phasing the array.
    """

    beam_rolls = []
    ref_z = (np.max(ant_z)+np.min(ant_z))/2

    n_phased = len(channel_list)

    for angle in phasing_angles:
        subbeam_rolls = {}
        for z, channel_id in zip(ant_z, range(n_phased)):
            delay = (z-ref_z)/0.3 * ref_index * np.sin(angle)
            roll = int(delay/time_step)
            subbeam_rolls[channel_id] = roll
        beam_rolls.append(subbeam_rolls)

        return beam_rolls

array_type = args.array

if (array_type == 'ARA'):
    min_freq = 130*units.MHz
    max_freq = 750*units.MHz
    sampling_rate = 1.5*units.GHz
    time_step = 1./sampling_rate
    window_width = 16
    only_primary = False
    primary_angles = default_angles
    secondary_angles = default_sec_angles
    ant_z_primary = [-46.5, -47.5, -48.5, -49.5, -50.5, -51.5, -52.5, -53.5] # primary antennas positions
    primary_channels = [0, 1, 2, 3, 4, 5, 6, 7] # channels used for primary beam
    beam_rolls = get_beam_rolls(ant_z_primary, primary_channels, primary_angles, time_step)
elif (array_type == 'RNO'):
    min_freq = 132*units.MHz
    max_freq = 700*units.MHz
    sampling_rate = 3*units.GHz
    time_step = 1./sampling_rate
    window_width = 32
    only_primary = True
    primary_angles = np.arcsin( np.linspace( np.sin(main_low_angle), np.sin(main_high_angle), 30) )
    ant_z_primary = [-98.5, -99.5, -100.5, -101.5] # primary antennas positions
    primary_channels = [0, 1, 2, 3] # channels used for primary beam
    beam_rolls = get_beam_rolls(ant_z_primary, primary_channels, primary_angles, time_step)

n_samples = 1000000 # number of samples
bandwidth = max_freq-min_freq
amplitude = (300 * 50 * constants.k * bandwidth / units.Hz) ** 0.5

Ntries = args.ntries # number of tries

threshold_factors = [2.15, 2.2, 2.25, 2.3, 2.35, 2.40]

if not only_primary:
    sec_channels = [0, 1, 3, 4, 6, 7] # channels used for secondary beam
    ant_z_secondary = [-46.5, -47.5, -49.5, -50.5, -52.5, -53.5] # secondary antennas positions
    dict_channels = {0:0, 1:1, 3:2, 4:3, 6:4, 7:5} # dictionary mapping primary channels to secondary channels
    sec_beam_rolls = get_beam_rolls(ant_z_secondary, sec_channels, secondary_angles, time_step)
else:
    sec_channels = []
    ant_z_secondary = []
    dict_channels = {}
    sec_beam_rolls = [{} for roll in beam_rolls]

n_beams = len(primary_angles)


for threshold_factor in threshold_factors:
    prob_per_window = 0
    for Ntry in range(Ntries):
        noise_array = []

        for iant in range(len(primary_channels)):

            noise_array.append(channelGenericNoiseAdder.bandlimited_noise(min_freq, max_freq, n_samples, sampling_rate, amplitude, type='rayleigh'))

        for subbeam_rolls, sec_subbeam_rolls in zip(beam_rolls, sec_beam_rolls):

            noise = None

            for channel_id in range(len(primary_channels)):

                if(noise is None):
                    noise = np.roll(noise_array[channel_id], subbeam_rolls[channel_id])
                else:
                    noise += np.roll(noise_array[channel_id], subbeam_rolls[channel_id])

                if channel_id in sec_channels:

                    sec_channel_id = dict_channels[channel_id]
                    noise += np.roll(noise_array[channel_id], sec_subbeam_rolls[sec_channel_id])

            n_windows = int(n_samples/window_width)
            n_windows = int(2*n_windows - 1)

            n_phased = len(primary_channels) + len(sec_channels)

            strides = noise.strides
            windowed_traces = np.lib.stride_tricks.as_strided(noise, \
                              shape=(n_windows, window_width), \
                              strides=(int(window_width / 2) * strides[0], strides[0]))

            squared_mean = np.sum(windowed_traces ** 2 / window_width, axis=1)
            squared_mean_threshold = n_phased * threshold_factor**2 * amplitude**2
            mask = squared_mean > squared_mean_threshold

            # If a phased direction triggers, the whole phased array triggers.
            # The following formula is justified as long as the probability is small
            # and each direction triggers independently of the rest.
            prob_per_window += np.sum( mask * np.ones(len(mask)) )/(n_windows*Ntries)

    # The 2 comes from the use of overlapping windows
    trigger_frequency = prob_per_window / (window_width/2)
    print('Threshold factor: {:.2f}, Fraction of noise triggers: {:.8f}%, Noise trigger rate: {:.2f}'.format(threshold_factor, prob_per_window*100., trigger_frequency/units.Hz))



from __future__ import absolute_import, division, print_function  # , unicode_literals
from dash.dependencies import Input, Output, State
import dash_core_components as dcc
import dash_html_components as html
import dash
import plotly.graph_objs as go
import json
import numpy as np
import uuid
import glob
from NuRadioReco.utilities import units
from NuRadioReco.framework.parameters import stationParameters as stnp
from NuRadioReco.framework.parameters import channelParameters as chp
from flask import Flask, send_from_directory
from app import app
from apps import overview
from apps import traces
from apps import cosmic_rays
from apps import simulation
from apps.common import get_point_index
import apps.simulation
import os
import argparse
import dataprovider
import logging
import datetime
import webbrowser
from NuRadioReco.modules.base import module
logger = module.setup_logger(level=logging.INFO)

argparser = argparse.ArgumentParser(description="Starts the Event Display, which then can be accessed via a webbrowser")
argparser.add_argument('file_location', type=str, help="Path of folder or filename.")
argparser.add_argument('--open-window', const=True, default=False, action='store_const', help="Open the event display in a new browser tab on startup")
argparser.add_argument('--port', default=8080, help="Specify the port the event display will run on")

parsed_args = argparser.parse_args()
data_folder = os.path.dirname(parsed_args.file_location)
if os.path.isfile(parsed_args.file_location):
    starting_filename = parsed_args.file_location
else:
    starting_filename = None
if parsed_args.open_window:
    webbrowser.open('http://127.0.0.1:{}/'.format(parsed_args.port))

provider = dataprovider.DataProvider()


app.title = 'NuRadioViewer'

app.layout = html.Div([
    dcc.Location(id='url', refresh=False),
    html.Div(id='event-click-coordinator', children=json.dumps(None), style={'display': 'none'}),
    html.Div(id='user_id', style={'display': 'none'},
             children=json.dumps(None)),
    html.Div(id='event-ids',  style={'display': 'none'},
             children=json.dumps([])),
    html.Div([
        html.Div([
            html.Div([
                html.Div([
                html.Div('File location:', className='input-group-text')
                ], className='input-group-addon'),
                dcc.Input(id='datafolder', placeholder='filename', type='text', value=data_folder, className='form-control')
                ], className='input-group'),
            html.Div([
                dcc.Dropdown(id='filename',
                             options=[],
                             multi=False,
                             value=starting_filename,
                             className='custom-dropdown'),
                 html.Div([
                    html.Button('open file', id='btn-open-file', className='btn btn-default')
                    ], className='input-group-btn'),
            ], className='input-group'),
            html.Div([
                html.Div([
                        html.Button([
                                html.Div(className='icon-arrow-left')
                            ],
                            id='btn-previous-event',
                            className='btn btn-primary',
                            n_clicks_timestamp = 0
                        ),
                        html.Button(
                            id = 'event-number-display',
                            children = '''''',
                            className='btn btn-primary'
                        ),
                        html.Button([
                                html.Div(className='icon-arrow-right')
                            ],
                            id='btn-next-event',
                            className='btn btn-primary',
                            n_clicks_timestamp=0
                            )
                    ],
                    className='btn-group',
                    style={'margin': '10px'}
                    ),
                html.Div([
                     dcc.Slider(
                        id='event-counter-slider',
                        step=1,
                        value=0,
                        marks={}
                    ),
                ],
                style={
                'padding': '10px 30px 20px',
                'overflow': 'hidden',
                'flex': '1'}
                ),
                html.Div([
                    dcc.Dropdown(
                        id='station-id-dropdown',
                        options = [],
                        multi=False
                    )
                ],
                style={'flex': 'none', 'padding': '10px', 'min-width': '200px'})
            ],
            style={
            'display': 'flex'
            })
        ], style={'flex': '7'}),
        html.Div([
            html.Div([
                html.Div('Run:', className='custom-table-td'),
                html.Div('', className='custom-table-td-last', id='event-info-run')
            ], className='custom-table-row'),
            html.Div([
                html.Div('Event:', className='custom-table-td'),
                html.Div('', className='custom-table-td-last', id='event-info-id')
            ], className='custom-table-row'),
            html.Div([
                html.Div('Time:', className='custom-table-td'),
                html.Div('', className='custom-table-td-last', id='event-info-time')
            ], className='custom-table-row')
        ], style={'flex': '1'}, className='event-info-table')
    ], style={'display': 'flex'}),
    dcc.RadioItems(
        options=[
            {'label': 'Summary', 'value': 'summary'},
            {'label': 'Traces', 'value': 'traces'},
            {'label': 'Simulation', 'value': 'simulation'},
            {'label': 'Cosmic Rays', 'value': 'cosmic_rays'}
        ],
        value='summary',
        id='content-selector',
        className='radio-content-selector',
        style={'background-color': '#f9f9f9'},
        labelStyle={'flex': '1', 'padding': '5px 50px'}

    ),
    html.Div('', id='content')
])

@app.callback(
Output('content', 'children'),
[Input('content-selector', 'value')]
)
def get_page_content(selection):
    if selection == 'summary':
        return [overview.layout]
    if selection == 'traces':
        return [traces.layout]
    if selection == 'simulation':
        return [simulation.layout]
    if selection == 'cosmic_rays':
        return [cosmic_rays.layout]
    return []

# next/previous buttons
@app.callback(
Output('event-counter-slider', 'value'),
[Input('btn-next-event', 'n_clicks_timestamp'),
Input('btn-previous-event', 'n_clicks_timestamp'),
Input('event-click-coordinator', 'children'),
Input('filename', 'value')],
[State('event-counter-slider', 'value'),
State('user_id', 'children')]
)
def set_event_number(next_evt_click_timestamp, prev_evt_click_timestamp, j_plot_click_info, filename, i_event, juser_id):
    context = dash.callback_context
    if filename is None:
        return 0
    if context.triggered[0]['prop_id'] == 'filename.value':
        return 0
    if context.triggered[0]['prop_id'] == 'event-click-coordinator.children':
        if context.triggered[0]['value'] is None:
            return 0
        return json.loads(context.triggered[0]['value'])['event_i']
    else:
        if context.triggered[0]['prop_id'] != 'btn-next-event.n_clicks_timestamp' and context.triggered[0]['prop_id'] != 'btn-previous-event.n_clicks_timestamp':
            return 0
        if context.triggered[0]['prop_id'] == 'btn-previous-event.n_clicks_timestamp':
            if i_event == 0:
                return 0
            else:
                return i_event - 1
        if context.triggered[0]['prop_id'] == 'btn-next-event.n_clicks_timestamp':
            user_id = json.loads(juser_id)

            number_of_events = provider.get_arianna_io(user_id, filename).get_n_events()
            if number_of_events == i_event + 1:
                return number_of_events -1
            else:
                return i_event + 1


@app.callback(
Output('event-number-display', 'children'),
[Input('filename', 'value'),
Input('event-counter-slider', 'value')]
)
def set_event_number_display(filename, event_number):
    if filename is None:
        return 'No file selected'
    return 'Event {}'.format(event_number)

@app.callback(
    Output('event-counter-slider', 'max'),
    [Input('filename', 'value')],
    [State('user_id', 'children')])
def update_slider_options(filename, juser_id):
    if filename is None:
        return 0
    user_id = json.loads(juser_id)
    ariio = provider.get_arianna_io(user_id, filename)
    number_of_events = ariio.get_n_events()
    return number_of_events - 1

@app.callback(
Output('event-counter-slider', 'marks'),
[Input('filename', 'value')],
[State('user_id', 'children')]
)
def update_slider_marks(filename, juser_id):
    if filename is None:
        return {}
    user_id = json.loads(juser_id)
    ariio = provider.get_arianna_io(user_id, filename)
    n_events = ariio.get_n_events()
    step_size = int(np.power(10., int(np.log10(n_events))))
    marks = {}
    for i in range(0, n_events, step_size):
        marks[i] = str(i)
    if n_events%step_size != 0:
        marks[n_events] = str(n_events)
    return marks


@app.callback(Output('user_id', 'children'),
              [Input('url', 'pathname')],
              [State('user_id', 'children')])
def set_uuid(pathname, juser_id):
    user_id = json.loads(juser_id)
    if(user_id is None):
        user_id = uuid.uuid4().hex
    return json.dumps(user_id)


@app.callback(Output('filename', 'options'),
              [Input('datafolder', 'value')])
def set_filename_dropdown(folder):
    return [{'label': l.split('/')[-1], 'value': l} for l in sorted(glob.glob(os.path.join(folder, '*.nur*')))]



@app.callback(Output('station-id-dropdown', 'options'),
            [Input('filename', 'value'),
            Input('event-counter-slider', 'value')],
            [State('user_id', 'children')])
def get_station_dropdown_options(filename, i_event, juser_id):
    if filename is None:
        return []
    user_id = json.loads(juser_id)
    ariio = provider.get_arianna_io(user_id, filename)
    event = ariio.get_event_i(i_event)
    dropdown_options = []
    for station in event.get_stations():
        dropdown_options.append({
            'label': 'Station {}'.format(station.get_id()),
            'value': station.get_id()
        })
    return dropdown_options

@app.callback(Output('station-id-dropdown', 'value'),
            [Input('filename', 'value'),
            Input('event-counter-slider', 'value')],
            [State('user_id', 'children')])
def set_to_first_station_in_event(filename, event_i, juser_id):
    if filename is None:
        return None
    user_id = json.loads(juser_id)
    ariio = provider.get_arianna_io(user_id, filename)
    event = ariio.get_event_i(event_i)
    for station in event.get_stations():
        return station.get_id()

# update event ids list from plot selection
@app.callback(Output('event-ids', 'children'),
              [Input('cr-skyplot', 'selectedData'),
               Input('cr-xcorrelation', 'selectedData'),
               Input('cr-xcorrelation-amplitude', 'selectedData'),
               Input('skyplot-xcorr', 'selectedData'),
               Input('cr-polarization-zenith', 'selectedData')],
              [State('event-ids', 'children')])
def set_event_selection(selectedData1, selectedData2, selectedData3, selectedData4, selectedData5, jcurrent_selection):
    current_selection = json.loads(jcurrent_selection)
    tcurrent_selection = []
    for i, selection in enumerate([selectedData1, selectedData2, selectedData3, selectedData4, selectedData5]):  # check which selection has fired the callback
        if selection is not None:
            event_ids = []
            for x in selection['points']:
                t = x['customdata']
                if t not in event_ids:
                    event_ids.append(t)
            if not np.array_equal(np.array(event_ids), current_selection):  # this selection has fired the callback
                tcurrent_selection = event_ids
    return json.dumps(tcurrent_selection)

def add_click_info(json_object, event_number_array, times_array):
    object = json.loads(json_object)
    if object is not None:
        event_number_array.append(object['event_i'])
        times_array.append(object['time'])

#finds out which one of the plots was clicked last (i.e. which one triggered the event update)
@app.callback(Output('event-click-coordinator', 'children'),
            [Input('cr-polarization-zenith', 'clickData'),
            Input('cr-skyplot', 'clickData'),
            Input('cr-xcorrelation', 'clickData'),
            Input('cr-xcorrelation-amplitude', 'clickData')])
def coordinate_event_click(cr_polarization_zenith_click, cr_skyplot_click, cr_xcorrelation_click, cr_xcorrelation_amplitude_click):
    context = dash.callback_context
    if context.triggered[0]['value'] is None:
        return None
    return json.dumps({
        'event_i': context.triggered[0]['value']['points'][0]['customdata'],
    })

@app.callback(Output('event-info-run', 'children'),
            [Input('event-counter-slider', 'value'),
            Input('filename', 'value')],
            [State('user_id', 'children')])
def update_event_info_run(event_i, filename, juser_id):
    if filename is None:
        return ""
    user_id = json.loads(juser_id)
    ariio = provider.get_arianna_io(user_id, filename)
    evt = ariio.get_event_i(event_i)
    return evt.get_run_number()

@app.callback(Output('event-info-id', 'children'),
            [Input('event-counter-slider', 'value'),
            Input('filename', 'value')],
            [State('user_id', 'children')])
def update_event_info_id(event_i, filename, juser_id):
    if filename is None:
        return ""
    user_id = json.loads(juser_id)
    ariio = provider.get_arianna_io(user_id, filename)
    evt = ariio.get_event_i(event_i)
    return evt.get_id()

@app.callback(Output('event-info-time', 'children'),
            [Input('event-counter-slider', 'value'),
            Input('filename', 'value'),
            Input('station-id-dropdown', 'value')],
            [State('user_id', 'children')])
def update_event_info_time(event_i, filename, station_id, juser_id):
    if filename is None or station_id is None:
        return ""
    user_id = json.loads(juser_id)
    ariio = provider.get_arianna_io(user_id, filename)
    evt = ariio.get_event_i(event_i)
    if evt.get_station(station_id).get_station_time() is None:
        return ''
    return '{:%d. %b %Y, %H:%M:%S}'.format(evt.get_station(station_id).get_station_time().datetime)

if __name__ == '__main__':
    if int(dash.__version__.split('.')[0]) <= 1:
        if int(dash.__version__.split('.')[1]) < 0:
            print('WARNING: Dash version 0.39.0 or newer is required, you are running version {}. Please update.'.format(dash.__version__))
    app.run_server(debug=False, port=parsed_args.port)



import argparse
import NuRadioReco.modules.io.coreas.readCoREAS
import NuRadioReco.modules.io.eventWriter
import NuRadioReco.framework.hybrid_shower
import NuRadioReco.detector.detector
import pickle
import datetime
import numpy
"""
To accomodate radio experiments that are part of a hybrid detector, NuRadioReco
can support custom detector classes and write them into  NuRadioReco event files.
This example shows what such a custom detector class should look like and how it
can be added to an event file.
"""
parser = argparse.ArgumentParser(description='Example showing how a custom hybrid detector can be added to an event file.')
parser.add_argument('--inputfilename', type=str, help='path to CoREAS file', default='example_data/example_event.h5')
parser.add_argument('--detectorfilename', type=str, help='path to detector description', default='example_data/arianna_detector_db.json')
parser.add_argument('--stationID', type=int, help='ID of the station to simulate', default=32)
parser.add_argument('--outputfilename', type=str, help='name of the output file', default='custom_detector.nur')
args = parser.parse_args()

#setting up stuff we need to read the CoREAS file
coreas_reader = NuRadioReco.modules.io.coreas.readCoREAS.readCoREAS()
coreas_reader.begin([args.inputfilename], args.stationID)
event_writer = NuRadioReco.modules.io.eventWriter.eventWriter()
event_writer.begin(args.outputfilename)
detector = NuRadioReco.detector.detector.Detector(args.detectorfilename)
detector.update(datetime.datetime(2019,1,1))


#define a custom detector class we want to store in the event
class CustomDetector():
    #the custom detector's creator must not require any arguments
    def __init__(self):
        self.__data = None

    #define some custom functions we want our detector to have
    def set_data(self, data):
        self.__data = data
    def get_data(self):
        return self.__data

    #the custom detector must know how to serialize and deserialize itself
    def serialize(self):
        data_pkl = pickle.dumps(self.__data, protocol=4)
        return data_pkl

    def deserialize(self, data_pkl):
        self.__data = pickle.loads(data_pkl)

for event in coreas_reader.run(detector):
    #This import is needed so we can read it later
    from CustomHybridDetector import CustomDetector
    #Create custom detector
    custom_detector = CustomDetector()
    #generate some data for the custom detector to store
    custom_data = numpy.random.random(10)
    custom_detector.set_data(custom_data)
    #create hybrid shower to hold the detector
    shower = NuRadioReco.framework.hybrid_shower.HybridShower('CustomShower')
    shower.set_hybrid_detector(custom_detector)
    #add shower to the hybrid information
    event.get_hybrid_information().add_hybrid_shower(shower)
    event_writer.run(event)



from NuRadioReco.modules.base.module import register_run
import NuRadioReco.framework.event
import NuRadioReco.framework.station
import NuRadioReco.framework.channel
import ROOT
import numpy as np
from NuRadioReco.utilities import units
import datetime


class readARIANNAData:
    """
    Assumes a tree with calibrated data, shifted by the stop. Very basic module for now.
    """

    def begin(self, input_file, station_id):
        self.data_tree = ROOT.TChain("CalibTree")
        self.data_tree.Add(input_file)
        self.calwv = ROOT.TSnCalWvData()
        self.data_tree.SetBranchAddress("AmpOutDataShifted.", self.calwv)

        self.raw = ROOT.TSnRawWaveform()
        self.data_tree.SetBranchAddress("RawData.", self.calwv)

        self.config_tree = ROOT.TChain("ConfigTree")
        self.config_tree.Add(input_file)
        self.readout_config = ROOT.TSnReadoutConfig()
        self.config_tree.SetBranchAddress("ReadoutConfig.", self.readout_config)

        self.n_events = self.data_tree.GetEntries()

        self._station_id = station_id
        self.__id_current_event = 0

        return self.n_events

    @register_run()
    def run(self):
        while True:
            if(self.__id_current_event >= self.n_events):
                # all events processed
                break
            self.data_tree.GetEntry(self.__id_current_event)
            self.config_tree.GetEntry(self.__id_current_event)

            evt_number = self.data_tree.EventHeader.GetEvtNum()
            run_number = self.data_tree.EventMetadata.GetRunNum()
            evt_triggered = self.data_tree.EventHeader.IsThermal()
            evt = NuRadioReco.framework.event.Event(run_number, evt_number)

            nChan = ord(self.readout_config.GetNchans())  # convert char to int
            self.sampling_rate = self.readout_config.GetSamplingRate() * units.GHz

            evt_time = datetime.datetime.fromtimestamp(self.data_tree.EventHeader.GetUnixTime())

            station = NuRadioReco.framework.station.Station(self._station_id)
            station.set_station_time(evt_time)
            station.set_triggered(evt_triggered)

            for iCh in range(nChan):
                channel = NuRadioReco.framework.channel.Channel(iCh)
                voltage = np.array(self.calwv.GetDataOnCh(iCh)) * units.mV
                channel.set_trace(voltage, self.sampling_rate)
                station.add_channel(channel)

            evt.set_station(station)
            self.__id_current_event += 1
            yield evt

    def end(self):
        pass



import numpy as np
import matplotlib.pyplot as plt
import logging
from NuRadioReco.utilities import units
import NuRadioReco.framework.channel
import NuRadioReco.framework.station
import NuRadioReco.framework.event
import NuRadioReco.modules.channelResampler
import NuRadioReco.modules.channelGenericNoiseAdder
import NuRadioReco.modules.channelBandPassFilter
import NuRadioReco.modules.channelSignalReconstructor
import NuRadioReco.modules.channelLengthAdjuster

import NuRadioReco.modules.ARA.triggerSimulator
import NuRadioReco.modules.trigger.highLowThreshold

from NuRadioReco.detector import detector
det = detector.Detector(json_filename='../example_data/dummy_detector.json')

from NuRadioReco.modules.base import module
logger = module.setup_logger(level=logging.WARNING)

channelGenericNoiseAdder = NuRadioReco.modules.channelGenericNoiseAdder.channelGenericNoiseAdder()
channelGenericNoiseAdder.begin(debug=False)
channelResampler = NuRadioReco.modules.channelResampler.channelResampler()
channelSignalReconstructor = NuRadioReco.modules.channelSignalReconstructor.channelSignalReconstructor()
channelSignalReconstructor.begin(debug=False)
channelBandPassFilter = NuRadioReco.modules.channelBandPassFilter.channelBandPassFilter()
channelLengthAdjuster = NuRadioReco.modules.channelLengthAdjuster.channelLengthAdjuster()
channelLengthAdjuster.begin(number_of_samples=400, offset=50)

from NuRadioReco.framework.parameters import stationParameters as stnp
from NuRadioReco.framework.parameters import channelParameters as chp

triggerSimulator_ARA = NuRadioReco.modules.ARA.triggerSimulator.triggerSimulator()
triggerSimulator_ARA.begin()
triggerSimulator_ARIANNA = NuRadioReco.modules.trigger.highLowThreshold.triggerSimulator()
triggerSimulator_ARIANNA.begin()

event_ARA = NuRadioReco.framework.event.Event(1,1)
event_ARIANNA = NuRadioReco.framework.event.Event(1,1)

station_ARA = NuRadioReco.framework.station.Station(101)
channel_ARA = NuRadioReco.framework.channel.Channel(0)

station_ARIANNA = NuRadioReco.framework.station.Station(101)
channel_ARIANNA = NuRadioReco.framework.channel.Channel(0)

#Switch between cosmic ray pulse (CoREAS) and Askaryan parameterization
CR = False

# TYPE_SNR = 'integrated_power'
TYPE_SNR = 'peak_amplitude'

if CR:
    # Use numpy array with band-pass limited pulse [30-1000] MHz, samples at 10 GHz
    data = np.load("example_data/Test_data_8.npy")
    test_pulse = data[2,:]
    # Normalize test pulse to 1
else:
    # PARAMEETERS for Askaryan Pulse
    energy = 1e19*units.eV
    fhad = 0.5
    viewing_angle = 54 * units.degree
    n_samples = 2**12
    dt = 0.1 *units.ns
    n_index = 1.5
    R = 100 * units.m

    from NuRadioMC.SignalGen import parametrizations as signalgen
    test_pulse = signalgen.get_time_trace(energy* fhad,viewing_angle, n_samples, dt, 'HAD', n_index, R, 'Alvarez2000')



test_pulse /= np.max(np.abs(test_pulse))

n_scaling = 50
result_ARA = np.zeros((n_scaling,2))
result_ARIANNA = np.zeros((n_scaling,2))

i = 0
n_iter = 20

for scaling in np.linspace(10*units.mV,200*units.mV,n_scaling):
    test_pulse_sc = test_pulse * scaling

    n_trigger_ARA = 0
    n_trigger_ARIANNA = 0

    SNR_ARA = 0
    SNR_ARIANNA = 0

    max = []
    for n in range(n_iter):

        channel_ARA.set_trace(test_pulse_sc,10*units.GHz)
        station_ARA.add_channel(channel_ARA)


        channel_ARIANNA.set_trace(test_pulse_sc,10*units.GHz)
        station_ARIANNA.add_channel(channel_ARIANNA)

        channelBandPassFilter.run(event_ARIANNA, station_ARIANNA, det, passband=[50 * units.MHz, 1000 * units.MHz],
            filter_type='rectangular')
        channelBandPassFilter.run(event_ARA, station_ARA, det, passband=[50 * units.MHz, 1000 * units.MHz],
            filter_type='rectangular')

        channelGenericNoiseAdder.run(event_ARA,station_ARA,
                            det,amplitude=20*units.mV,min_freq=50*units.MHz,max_freq=1000*units.MHz,type='perfect_white')
        channelGenericNoiseAdder.run(event_ARIANNA,station_ARIANNA,
                            det,amplitude=20*units.mV,min_freq=50*units.MHz,max_freq=1000*units.MHz,type='perfect_white')


        channelResampler.run(event_ARA, station_ARA, det, sampling_rate=1 * units.GHz)
        channelResampler.run(event_ARIANNA, station_ARIANNA, det, sampling_rate=1 * units.GHz)

        channelSignalReconstructor.run(event_ARIANNA, station_ARIANNA, det)
        channelSignalReconstructor.run(event_ARA, station_ARA, det)

        channelLengthAdjuster.run(event_ARIANNA, station_ARIANNA, det)
        channelLengthAdjuster.run(event_ARA, station_ARA, det)

        triggerSimulator_ARA.run(event_ARA, station_ARA, det, power_threshold=6.5,
                                        coinc_window = 110 * units.ns,
                                        number_concidences =  1,
                                        triggered_channels = [0, 1, 2, 3, 4, 5, 6, 7])

        triggerSimulator_ARIANNA.run(event_ARIANNA, station_ARIANNA, det,threshold_high=36 * units.mV,
                threshold_low=-36 * units.mV,
                high_low_window=20 * units.ns,
                coinc_window=32 * units.ns,
                number_concidences=1,
                triggered_channels=[0, 1, 2, 3])

#
        SNR_ARA +=  station_ARA.get_channel(0)[chp.SNR][TYPE_SNR]

        SNR_ARIANNA += station_ARIANNA.get_channel(0)[chp.SNR][TYPE_SNR]

        max.append(np.max(np.abs(station_ARA.get_channel(0).get_trace())))
#         channel_ARA.clear_trace()

        if station_ARA.has_triggered() != False:
            n_trigger_ARA  += 1.
        if station_ARIANNA.has_triggered()!= False:
            n_trigger_ARIANNA  += 1.


    result_ARA[i,0] = SNR_ARA / n_iter
    result_ARA[i,1] = n_trigger_ARA / n_iter

    result_ARIANNA[i,0] = SNR_ARIANNA / n_iter
    result_ARIANNA[i,1] = n_trigger_ARIANNA / n_iter
    i += 1




plt.figure()
plt.plot(result_ARA[:,0],result_ARA[:,1], linestyle='None', marker ='o',label="ARA, power_threshold 6.5")
plt.plot(result_ARIANNA[:,0],result_ARIANNA[:,1], linestyle='None', marker ='s',label="ARIANNA, 3 sigma")
plt.ylabel("Trigger efficiency on one antenna")
plt.xlabel(TYPE_SNR)
plt.legend()
plt.show()


from NuRadioReco.modules.base.module import register_run
import numpy as np
import fractions
from scipy import signal
from decimal import Decimal
from NuRadioReco.utilities import units
import logging
logger = logging.getLogger('stationResampler')


class electricFieldResampler:
    """
    resamples the electric field trace to a new sampling rate
    """

    def __init__(self):
        self.begin()

    def begin(self, debug=False):
        self.__max_upsampling_factor = 5000
        self.__debug = debug
        pass

    @register_run()
    def run(self, event, station, det, sampling_rate):
        """
        resample electric field

        Parameters
        ----------
        event: event

        station: station

        det: detector

        sampling_rate: float
            desired new sampling rate

        """
        # access simulated efield and high level parameters
        # calculate sampling and FFT resolution
        for efield in station.get_electric_fields():
            orig_binning = 1. / efield.get_sampling_rate()

            target_binning = 1. / sampling_rate
            resampling_factor = fractions.Fraction(Decimal(orig_binning / target_binning)).limit_denominator(self.__max_upsampling_factor)
            logger.debug("resampling channel trace by a factor of {}. Original binning {:.3g}ns, target binning {:.3g}".format(resampling_factor,
                                                                                                                                orig_binning / units.ns,
                                                                                                                                target_binning / units.ns))
            new_length = int(efield.get_trace().shape[1] * resampling_factor)
            resampled_efield = np.zeros((3, new_length))  # create new data structure with new efield length

            for iE in range(3):
                trace = efield.get_trace()[iE]

                if(resampling_factor.numerator != 1):
                    trace = signal.resample(trace, resampling_factor.numerator * len(trace))  # , window='hann')
                if(resampling_factor.denominator != 1):
                    trace = signal.resample(trace, len(trace) // resampling_factor.denominator)  # , window='hann')

                resampled_efield[iE] = trace
            # prevent traces to get an odd number of samples. If the trae has an odd number of samples, the last sample is discarded.
            if resampled_efield.shape[-1] % 2 != 0:
                resampled_efield = resampled_efield.T[:-1].T
            efield.set_trace(resampled_efield, sampling_rate)

    def end(self):
        pass



from NuRadioReco.modules.base.module import register_run
from NuRadioReco.utilities import units
from NuRadioReco.framework.parameters import stationParameters as stnp
from NuRadioReco.framework.trigger import SimpleThresholdTrigger
from NuRadioReco.modules.trigger.highLowThreshold import get_majority_logic
import numpy as np
import time
import logging



def get_threshold_triggers(trace, threshold):
    """
    calculats a simple threshold trigger

    Parameters
    ----------
    trace: array of floats
        the signal trace
    threshold: float
        the threshold
    Returns
    -------
    triggered bins: array of bools
        the bins where the trigger condition is satisfied
    """

    return np.abs(trace) >= threshold


class triggerSimulator:
    """
    Calculate a very simple amplitude trigger.
    """

    def __init__(self):
        self.__t = 0
        self.begin()
        self.logger = logging.getLogger('NuRadioReco.simpleThresholdTrigger')

    def begin(self, debug=False):
        self.__debug = debug

    @register_run()
    def run(self, evt, station, det,
            threshold=60 * units.mV,
            number_concidences=1,
            triggered_channels=None,
            coinc_window=200 * units.ns,
            trigger_name='default_simple_threshold'):
        """
        simulate simple trigger logic, no time window, just threshold in all channels

        Parameters
        ----------
        number_concidences: int
            number of channels that are requried in coincidence to trigger a station
        threshold: float
            threshold above (or below) a trigger is issued, absolute amplitude
        triggered_channels: array of ints or None
            channels ids that are triggered on, if None trigger will run on all channels
        coinc_window: float
            time window in which number_concidences channels need to trigger
        trigger_name: string
            a unique name of this particular trigger
        """
        t = time.time()

        sampling_rate = station.get_channel(0).get_sampling_rate()
        dt = 1. / sampling_rate
        triggerd_bins_channels = []
        if triggered_channels is None:
            for channel in station.iter_channels():
                channel_trace_start_time = channel.get_trace_start_time()
                break
        else:
            channel_trace_start_time = station.get_channel(triggered_channels[0]).get_trace_start_time()
        channels_that_passed_trigger = []
        for channel in station.iter_channels():
            channel_id = channel.get_id()
            if triggered_channels is not None and channel_id not in triggered_channels:
                self.logger.debug("skipping channel {}".format(channel_id))
                continue
            if channel.get_trace_start_time() != channel_trace_start_time:
                self.logger.warning('Channel has a trace_start_time that differs from the other channels. The trigger simulator may not work properly')
            trace = channel.get_trace()
            triggerd_bins = get_threshold_triggers(trace, threshold)
            triggerd_bins_channels.append(triggerd_bins)
            if True in triggerd_bins:
                channels_that_passed_trigger.append(channel.get_id())

        has_triggered, triggered_bins, triggered_times = get_majority_logic(
            triggerd_bins_channels, number_concidences, coinc_window, dt)
        # set maximum signal aplitude
        max_signal = 0
        if(has_triggered):
            for channel in station.iter_channels():
                max_signal = max(max_signal, np.abs(channel.get_trace()[triggered_bins]).max())
            station.set_parameter(stnp.channels_max_amplitude, max_signal)
        trigger = SimpleThresholdTrigger(trigger_name, threshold, triggered_channels,
                                         number_concidences)
        trigger.set_triggered_channels(channels_that_passed_trigger)
        if has_triggered:
            trigger.set_triggered(True)
            trigger.set_trigger_time(triggered_times.min()+channel_trace_start_time)
            self.logger.debug("station has triggered")
        else:
            trigger.set_triggered(False)
            trigger.set_trigger_time(0)
            self.logger.debug("station has NOT triggered")
        station.set_trigger(trigger)

        self.__t += time.time() - t

    def end(self):
        from datetime import timedelta
        self.logger.setLevel(logging.INFO)
        dt = timedelta(seconds=self.__t)
        self.logger.info("total time used by this module is {}".format(dt))
        return dt



from NuRadioReco.utilities import units
from NuRadioReco.modules.base.module import register_run
import numpy as np
import time
import logging
import scipy.signal
from NuRadioReco.modules.channelGenericNoiseAdder import channelGenericNoiseAdder
from NuRadioReco.framework.trigger import IntegratedPowerTrigger
from NuRadioReco.utilities.diodeSimulator import diodeSimulator
import NuRadioReco.framework.channel

logger = logging.getLogger('ARAtriggerSimulator')


class triggerSimulator:
    """
    Calculates the trigger of an event.
    Uses the ARA trigger logic of a tunnel diode.
    Implementation similar to PyRex by Ben Hokanson-Fasig/
    """

    def __init__(self):
        self.__t = 0
        self._power_mean = None
        self._power_std = None
        self._diode = diodeSimulator()
        logger.warning("This module does not contain cutting the trace to ARA specific parameters.")

    def has_triggered(self, channel):
        """
        Check if the detector system triggers on a given channel.
        Passes the signal through the tunnel diode. Then compares the maximum
        and minimum values to a tunnel diode noise signal. Triggers if one of
        the maximum or minimum values exceed the noise mean +/- the noise rms
        times the power threshold.
        Parameters
        ----------
        channel : Channel
            ``Channel`` object on which to test the trigger condition.
        Returns
        -------
        boolean
            Whether or not the antenna triggers on `channel`.
        """

        # Send signal through tunnel_diode
        after_tunnel_diode = self._diode.tunnel_diode(channel)
        low_trigger = (self._power_mean -
                       self._power_std * np.abs(self.power_threshold))

        return np.min(after_tunnel_diode) < low_trigger

    @register_run()
    def run(self, evt, station, det,
            power_threshold=6.5,
            coinc_window=110 * units.ns,
            number_concidences=3,
            triggered_channels=[0, 1, 2, 3, 4, 5, 6, 7],
            power_mean=None,
            power_std=None,
            trigger_name='default_integrated_power'):
        """
        simulate ARA trigger logic

        Parameters
        ----------
        power_threshold: float
            The factor of sigma that the signal needs to exceed the noise
        coinc_window: float
            time window in which number_concidences channels need to trigger
        number_concidences: int
            number of channels that are requried in coincidence to trigger a station
        triggered_channels: array of ints
            channels ids that are triggered on
        power_mean : float
            Parameter extracted in ARA from noise.
            If not given, it is calculated from generic noise
        power_std : float
            Parameter extracted in ARA from noise.
            If not given, it is calculated from generic noise
        trigger_name: string
            a unique name of this particular trigger
        """
        # if the run method specifies power mean and rms we use these values,
        # if the parameters are None, the power mean and rms gets calculated for
        # some standard assumptions on the noise RMS and it needs to be done only once
        if(power_mean is not None and power_std is not None):
            self._power_mean = power_mean
            self._power_std = power_std
        else:
            error_msg  = 'The power_mean and power_std parameters are not defined. '
            error_msg += 'Please define them. You can use the calculate_noise_parameters '
            error_msg += 'function in utilities.diodeSimulator to do so.'
            raise ValueError(error_msg)

        self.power_threshold = power_threshold

        # No coincidence requirement yet
        trigger = {}
        trigger_times = []
        times_min = []
        times_max = []
        sampling_rates = []
        number_triggered_channels = 0

        for channel in station.iter_channels():
            channel_id = channel.get_id()
            if channel_id not in triggered_channels:
                continue
            trigger[channel_id] = self.has_triggered(channel)
            if trigger[channel_id]:
                number_triggered_channels += 1
                times = channel.get_times()
                trace_after_diode = self._diode.tunnel_diode(channel)
                arg_trigger = np.argmin(trace_after_diode)
                trigger_times.append(times[arg_trigger])
                times_min.append(np.min(times))
                times_max.append(np.max(times))
                sampling_rates.append(channel.get_sampling_rate())

        has_triggered = False
        trigger_time = None

        if (number_triggered_channels >= number_concidences):

            trace_times = np.arange(np.min(times_min), np.max(times_max),
                                    1/np.min(sampling_rates))

            trigger_times = np.array(trigger_times)
            slice_left = int(coinc_window/2/(trace_times[1]-trace_times[0]))
            slice_right = len(trace_times)-slice_left
            for trace_time in trace_times[slice_left:slice_right]:
                if ( np.sum( np.abs(trace_time-trigger_times) <= coinc_window/2 ) >= number_concidences ):
                    has_triggered = True
                    trigger_time = np.min(trigger_times)
                    break

        trigger = IntegratedPowerTrigger(trigger_name, power_threshold,
                                         coinc_window, channels=triggered_channels,
                                         number_of_coincidences=number_concidences,
                                         power_mean=self._power_mean, power_std=self._power_std)

        if not has_triggered:
            trigger.set_triggered(False)
            logger.info("Station has NOT passed trigger")
            trigger_time = 0
            trigger.set_trigger_time(trigger_time)
        else:
            trigger.set_triggered(True)
            trigger.set_trigger_time(trigger_time)
            logger.info("Station has passed trigger, trigger time is {:.1f} ns (sample {})".format(
                trigger.get_trigger_time() / units.ns, trigger_time))

        station.set_trigger(trigger)

    def end(self):
        from datetime import timedelta
        logger.setLevel(logging.INFO)
        dt = timedelta(seconds=self.__t)
        logger.info("total time used by this module is {}".format(dt))
        return dt



"""
This file calculates the SNR curves for a given phased array configuration.
Neutrinos specified by the input file should come from a fixed arrival direction
if the SNR curve for that particular direction is required. It is convenient
that the array sees the neutrino at the Cherenkov angle, so that the signals
are clearer. The signals are then rescaled to have the appropriate SNR and
the phased trigger is run to decide whether the event is selected.

The phased array configuration
in this file is inspired by the deployed ARA phased array: 1.5 GS/s, 8 antennas
at a depth of ~50 m, 15 phasing directions with primary and secondary beams.
In order to run, we need a detector file and a configuration file, included in
this folder. To run the code, type:

python T02RunSNR.py input_neutrino_file.hdf5 proposalcompact_50m_1.5GHz.json
config.yaml output_NuRadioMC_file.hdf5 output_SNR_file.hdf5 output_NuRadioReco_file.nur(optional)

The output_SNR_file.hdf5 contains the information on the SNR curves stored in
hdf5 format. The fields in this file are:
    - 'total_events', the total number of events that have been used
    - 'SNRs', a list with the SNR

The antenna positions can be changed in the detector position. The config file
defines de bandwidth for the noise RMS calculation. The properties of the phased
array can be changed in the current file - phasing angles, triggering channels,
bandpass filter and so on.

WARNING: this file needs NuRadioMC to be run.
"""

from __future__ import absolute_import, division, print_function
import argparse
# import detector simulation modules
import NuRadioReco.modules.efieldToVoltageConverter
import NuRadioReco.modules.trigger.simpleThreshold
import NuRadioReco.modules.phasedarray.triggerSimulator
import NuRadioReco.modules.channelResampler
import NuRadioReco.modules.channelBandPassFilter
import NuRadioReco.modules.channelGenericNoiseAdder
from NuRadioReco.utilities import units
from NuRadioMC.simulation import simulation
import json
import numpy as np
import logging
from NuRadioReco.modules.base import module
import logging
logger = module.setup_logger(level=logging.WARNING)

# initialize detector sim modules
efieldToVoltageConverter = NuRadioReco.modules.efieldToVoltageConverter.efieldToVoltageConverter()
efieldToVoltageConverter.begin(debug=False)
triggerSimulator = NuRadioReco.modules.phasedarray.triggerSimulator.triggerSimulator()
channelResampler = NuRadioReco.modules.channelResampler.channelResampler()
channelBandPassFilter = NuRadioReco.modules.channelBandPassFilter.channelBandPassFilter()
channelGenericNoiseAdder = NuRadioReco.modules.channelGenericNoiseAdder.channelGenericNoiseAdder()
thresholdSimulator = NuRadioReco.modules.trigger.simpleThreshold.triggerSimulator()

N = 51
SNRs = np.linspace(0.5,5,N)
SNRtriggered = np.zeros(N)
def count_events():
    count_events.events += 1
count_events.events = 0

class mySimulation(simulation.simulation):

    def _detector_simulation(self):
        # start detector simulation
        efieldToVoltageConverter.run(self._evt, self._station, self._det)  # convolve efield with antenna pattern
        # downsample trace back to detector sampling rate
        new_sampling_rate = 1.5 * units.GHz
        channelResampler.run(self._evt, self._station, self._det, sampling_rate=new_sampling_rate)

        reject_event = False
        threshold_cut = True
        # Forcing a threshold cut BEFORE adding noise for limiting the noise-induced triggers
        if threshold_cut:

            thresholdSimulator.run(self._evt, self._station, self._det,
                                 threshold=1 * self._Vrms,
                                 triggered_channels=None,  # run trigger on all channels
                                 number_concidences=1,
                                 trigger_name='simple_threshold')

            if not self._station.get_trigger('simple_threshold').has_triggered():
                reject_event = True
                print("You shall not pass (the threshold)")

        # Copying the original traces. Not really needed
        original_traces = {}

        for channel in self._station.iter_channels():  # loop over all channels (i.e. antennas) of the station

            trace = np.array( channel.get_trace() )
            channel_id = channel.get_id()
            original_traces[channel_id] = trace

        # Finding bins for each channel so as to calculate the peak to peak voltage
        ext_bins = {}

        for channel in self._station.iter_channels():  # loop over all channels (i.e. antennas) of the station

            trace = np.array( channel.get_trace() )
            channel_id = channel.get_id()
            max_bin = np.argwhere( trace == np.max(trace) )[0,0]
            min_bin = np.argwhere( trace == np.min(trace) )[0,0]
            if ( max_bin < min_bin ):
                left_bin = max_bin - 20
                right_bin = min_bin + 20
            else:
                left_bin = min_bin - 20
                right_bin = max_bin + 20
            ext_bins[channel_id] = (left_bin, right_bin)

        # Calculating peak to peak voltage and the necessary factors for the SNR.
        Vpps = []
        for channel in self._station.iter_channels():  # loop over all channels (i.e. antennas) of the station

            left_bin, right_bin = ext_bins[channel.get_id()]
            trace = np.array( channel.get_trace() )
            try:
                Vpp = np.max(trace[left_bin:right_bin])-np.min(trace[left_bin:right_bin])
            except:
                Vpp = 0
                #reject_event = True
            Vpps.append(Vpp)

        factor = 1./(np.mean(Vpps)/2/self._Vrms)
        #factor = 0
        mult_factors = factor * SNRs
        print(factor)

        if True in mult_factors > 1.e10:
            reject_event = True

        for factor, iSNR in zip(mult_factors, range(len(mult_factors))):

            for channel in self._station.iter_channels():  # loop over all channels (i.e. antennas) of the station

                trace = original_traces[channel.get_id()][:] * factor
                channel.set_trace( trace, sampling_rate = new_sampling_rate )

            min_freq = 100 * units.MHz
            max_freq = 750 * units.MHz
            Vrms_ratio = ((max_freq-min_freq) / self._cfg['trigger']['bandwidth'])**2
            # Adding noise AFTER the SNR calculation
            channelGenericNoiseAdder.run(self._evt, self._station, self._det, amplitude=self._Vrms*Vrms_ratio,
                                         min_freq=min_freq,
                                         max_freq=max_freq,
                                         type='rayleigh')

            # bandpass filter trace, the upper bound is higher then the sampling rate which makes it just a highpass filter
            channelBandPassFilter.run(self._evt, self._station, self._det, passband=[130 * units.MHz, 1000 * units.GHz],
                                      filter_type='butter', order=2)
            channelBandPassFilter.run(self._evt, self._station, self._det, passband=[0, 750 * units.MHz],
                                      filter_type='butter', order=10)

            # Phased array with ARA-like power trigger
            has_triggered = triggerSimulator.run(self._evt, self._station, self._det,
                                 threshold= 2.5 * self._Vrms,
                                 triggered_channels=None,  # run trigger on all channels
                                 secondary_channels=[0,1,3,4,6,7], # secondary channels
                                 trigger_name='primary_and_secondary_phasing',
                                 set_not_triggered=(not self._station.has_triggered("simple_threshold")),
                                 ref_index=1.78)

            if (has_triggered and not reject_event):
                print('Trigger for SNR', SNRs[iSNR])
                SNRtriggered[iSNR] += 1

        if not reject_event:
            count_events()
            print(count_events.events)
            print(SNRtriggered)

parser = argparse.ArgumentParser(description='Run NuRadioMC simulation')
parser.add_argument('inputfilename', type=str,
                    help='path to NuRadioMC input event list')
parser.add_argument('detectordescription', type=str,
                    help='path to file containing the detector description')
parser.add_argument('config', type=str,
                    help='NuRadioMC yaml config file')
parser.add_argument('outputfilename', type=str,
                    help='hdf5 output filename')
parser.add_argument('outputSNR', type=str,
                    help='outputfilename for the snr files')
parser.add_argument('outputfilenameNuRadioReco', type=str, nargs='?', default=None,
                    help='outputfilename of NuRadioReco detector sim file')
args = parser.parse_args()

sim = mySimulation(inputfilename=args.inputfilename,
                            outputfilename=args.outputfilename,
                            detectorfile=args.detectordescription,
                            outputfilenameNuRadioReco=args.outputfilenameNuRadioReco,
                            config_file=args.config)
sim.run()

print("Total events", count_events.events)
print("SNRs: ", SNRs)
print("Triggered: ", SNRtriggered)

output = {}
output['total_events'] = count_events.events
output['SNRs'] = list( SNRs )
output['triggered'] = list( SNRtriggered )

outputfile = args.outputSNR
with open(outputfile, 'w+') as fout:

    json.dump(output, fout, sort_keys=True, indent=4)
